function H_SISO = MTLHarnessTF(f, nodes, edges, wires, tx, rx, ...
    txWire, rxWire, Y_L, Y_TX, gndProp)
%MTLHARNESSTF computes the TF between any two points in a harness
%
% SYNOPSIS H_SISO = MTLHarnessTF(f, nodes, edges, wires, txId, rxId, Y_L,
% Y_TX, gndProp)
%
% INPUT f: frequencies at which to evaluate the TF
%       nodes: cell array with names of the nodes
%       edges: structs describing the edges between the junctions
%       tx: name of transmitter node
%       rx: name of receiver node
%       txWire: wire used for transmitting
%       rxWire: wire used for receiving
%       Y_L: load admittance at the nodes
%       Y_TX: input admittance matrix at the transmitter
%       gndProp: struct describing the ground plane (if any)
%
% OUTPUT: H_SISO: SISO TF from the txWire at the transmitter to rxWire
% wire at the receiver
%
%

%default for nodes per unit of distance when computing PUL parameters
NODESPERUNIT = 5000;

%set this to true if you want random loads at all nodes whose name starts
%with RNDPREFIX
RANDOMLOAD=false;

if (RANDOMLOAD)
    
    %the final impedance will be [RMIN, RMAX] +- i*[IMIN, IMAX], where 
    %[X, Y] denotes any integer number between X and Y
    
    RNDPREFIX = 'C'; %nodes starting with RNDPREFIX should get random loads
    rndP.RMIN = 10; %minimum of real part of impedance
    rndP.RMAX = 1000; %maximum of real poart of impedance
    rndP.IMIN = 10; %minimum of imaginary part of impedance
    rndP.IMAX = 1000; %maximum of imaginary part of impedance
    rndP.DIFFLOADPROB = 20; %probability for a differential load
end

%get IDs of transmitter and receiver
txId = find(strcmp(tx, nodes), 1);
rxId = find(strcmp(rx, nodes), 1);

%check if transmitter and receiver were found in "nodes"
if (isempty(txId))
    error('MTLHarnessTF:txNotFound', ['Transmitter "' tx ...
        '" not found in list of nodes']);
end
if (isempty(rxId))
    error('MTLHarnessTF:rxNotFound', ['Receiver "' rx ...
        '" not found in list of nodes']);
end

%some sanity checks
if (~iscell(nodes)) %nodes is expected to be cell array of strings
    error('expected nodes to be cell array');
end
if (~isstruct(edges))
    error('expected edges to be a struct array');
end
if (~isstruct(wires))
    error('expected wires to be a struct array');
end

%parse the load admittance input:
%Y_L may either be: 
% - a scalar: all loads are set to that scalar, between every wire and 
% ground.
% - an array: must be length(nodes). each scalar in the array is assigned to
% a junction, and all loads at the junction are set to that value
% - a cell array: see below
if ( ~iscell(Y_L) )
    %if Y_L isn't a cell, it is either a scalar or a vector
    if (isscalar(Y_L))
        Y_L = num2cell(ones(1, length(nodes)) * Y_L);
    elseif (isvector(Y_L))
        Y_L = num2cell(Y_L);
    else
        error(['if Y_L is not a cell array, it must be either a scalar' ...
            ' or a vector']);
    end
end

%check if Y_L contains an entry for every node
if ( (length(nodes) ~= length(Y_L)) )
    error(['if Y_L is a (cell) array, its length must be equal to the ' ...
        'number of junctions.']);
end

%create the tree structure from the nodes and edges
disp('creating tree');
[tree edges] = createTree(nodes, edges, nodes{1});

%if gndProp isn't given, no ground plane will be used
if (exist('gndProp', 'var'))
    gndPlane = true;
else
    gndPlane = false;
    gndProp = [];
end

%do the wire assingment
disp('assigning individual wires to edges');
[tree edgesParam] = assignWires(tree, edges, wires, gndPlane, gndProp);



disp('parsing load admittances');
Y_Ls = cell(1, length(tree));
%The load admittance matrix needs to define the loads for all ports at a
%node. Notice however that some nodes have additional, virtual ports. (See
%assignWires.m). This needs to be taken into account.

if (RANDOMLOAD)
    disp('MTLHarnessTF: Warning: Using randomized loads for all nodes that begin with "C"');
end

for k = 1:length(tree)
    %number of real (=physical) ports at that node
    numRealPorts = length(tree(k).ports);
    %Y_L{k} may either be...
    inputY_L = Y_L{k};
    if (isa(inputY_L, 'function_handle')) %function handle
        %the real ports are always the first ports
        currY_L = inputY_L(numRealPorts, f);
    elseif (isscalar(inputY_L)) %scalar
        currY_L = repmat(inputY_L*eye(numRealPorts), [1 1 length(f)]); 
    elseif (isvector(inputY_L)) %vector
        
        %sanity check
        if (length(inputY_L) ~= numRealPorts) %...of length numPorts
            error(['load vector specified for node ' tree(k).descr ...
                '; expected ' num2str(numRealPorts) ' values, got ' ...
                num2str(length(inputY_L))]);
        end
        
        currY_L = repmat(diag(inputY_L), [1 1 length(f)]);
    else %matrix
        if ( 1 == size(inputY_L, 3) )
            inputY_L = repmat(inputY_L, [1 1 length(f)]);
        end
        
        %sanity check
        if (~isequal( ...
                [numRealPorts, numRealPorts, length(f)], ...
                size(inputY_L)))
            disp(['size of load admittance matrix for node ' tree(k).descr]);
            disp(size(inputY_L));
            disp(['expected ' num2str(numRealPorts) 'x' ...
                num2str(numRealPorts) 'x' num2str(length(f)) '.']);
            error('load admittance matrix dimension mismatch');
        end
        
        currY_L = inputY_L;
    end
    
    if (RANDOMLOAD && strncmp(tree(k).descr, RNDPREFIX, length(RNDPREFIX)))
        currY_L = genRndLoad(rndP, numRealPorts, f);
    end
    
    %load admittance matrix for all ports (including the virutal ones)
    portY_L = zeros(tree(k).numPorts, tree(k).numPorts, length(f));
    portY_L(1:numRealPorts, 1:numRealPorts, :) = currY_L;
    Y_Ls{k} = portY_L;
end


numEdges = length(edgesParam);

%electrical properties of each tube
edgesElProp = struct('f', {f}, 'Y', cell(1, numEdges), 'Z', cell(1, numEdges));

for k = 1:numEdges
    disp(['processing tube ' num2str(k) '/' num2str(numEdges)]);
    %creates a (nx2)-matrix for every tube with the (x,y)-coordinates of the
    %wires in the tube.
    [edgesParam(k).condPos lim] = ...
        genRndCondPos(edgesParam(k).condRad + edgesParam(k).dielThickness);
    %region of interest
    edgesParam(k).xmin = lim(1)*1.1;
    edgesParam(k).xmax = lim(2)*1.1;
    edgesParam(k).ymin = lim(3)*1.1;
    edgesParam(k).ymax = lim(4)*1.1;
    if (gndPlane)
        edgesParam(k).ymax = edgesParam(k).ymax + gndProp.gndDist;
    end
    %of course it is possible to enter the conductor positions and region
    %of interest manually.
    
    %computes a hash for the tube, based on the number of wires, the conductors
    %radius, the sheats thickness, and the sheaths relative permittivity.
    %Replace with your own hashing function that uses the properties you need
    %to identify your edges
    %compute hash for setup. If a tube already has a hashSuffix, append it.
    hash = sdbmTubeHash(edgesParam(k).NCond, edgesParam(k).condRad, ...
        edgesParam(k).dielThickness, edgesParam(k).e_r);
    hashSuffix = '';
    if (isfield(edgesParam, 'hashSuffix'))
        if (~isempty(edgesParam(k).hashSuffix))
            hashSuffix = ['_' edgesParam(k).hashSuffix];
        end
    end
    edgesParam(k).hash = [hash hashSuffix];
    
    %NOTE on the hashing: if you don't use randomized positions, it is
    %probably advisable to implement your own hashing function that
    %includes the positions of the conductors in the hash.
    
    
    %computes electrical properties of the tube. If you want to change the
    %algorithm with which the PUL parameters are computed, do so in this
    %function, or replace this function call with something else that
    %computes the characteristic impedance and admittance matrices for a
    %tube
    [Y Z] = tubeYZ(f, edgesParam(k), 'findepC', NODESPERUNIT);
    edgesElProp(k).Y = Y; %Y: nxnxnf, n: NCond-1, nf: number of frequencies
    edgesElProp(k).Z = Z; %Z: nxnxnf, n: NCond-1, nf: number of frequencies
end

%we need to know which edge is the first/last one so that we can identify
%where the txWire/rxWire connects to

%get a path through the tree (needed as we need to know the names of the
%nodes on the path)
[sPath, ~] = findPath([tree.parent], txId, rxId);

%get the edge that connects to the tx/rx
[~, txEdgeId] = findEdge(sPath(1), sPath(2), edges);
[~, rxEdgeId] = findEdge(sPath(end-1), sPath(end), edges);
%get the connection arrays at the tx/rx
[txConn, ~] = getConn(edgesParam(txEdgeId), sPath(1), sPath(2));
[~, rxConn] = getConn(edgesParam(rxEdgeId), sPath(end-1), sPath(end));
%get all wires that run through the edges
txEdgeWires = edgesParam(txEdgeId).wireIdx;
rxEdgeWires = edgesParam(rxEdgeId).wireIdx;
%find the indices of txWire/rxWire in the edges
txWireId = strcmp(txEdgeWires, txWire);
rxWireId = strcmp(rxEdgeWires, rxWire);
%lookup to which port the txWire/rxWire connect to at tx/rx
txPortId = txConn(txWireId);
rxPortId = rxConn(rxWireId);

%We assume that the receiver has a load of  50 ohm
rxWireY_L = 1/50;
disp(['rx wire admittance: ' num2str(rxWireY_L)]);
Y_Ls{rxId}(rxPortId, rxPortId, :) = rxWireY_L;

%and we assume that the transmitter has a load of 50 ohm
R_S = 50;
disp(['tx R_S: ' num2str(R_S)]);

disp('computing TF (this may take a while, depending on the size of "f")');
[Y_IN H_MIMO] = reduceTree(tree, edgesParam, edgesElProp, txId, rxId, Y_Ls);

%compute  the SIMO TF from the MIMO TF, using the input admittance
H_SIMO = singleLineTF(H_MIMO, Y_TX, Y_IN, R_S, txPortId);

H_SISO = H_SIMO(rxPortId, :);

end