function [tree tubes] = assignWires(tree, tubes, wires, gndPlane, gndProp)
%ASSIGNWIRES tries to figure out how wires could be connected in tubes
%
% SYNOPSIS [tree tubes] = assignWires(tree, tubes, wires, gndPlane,
% gndProp)
%
% INPUT tree: 
%           - tree.parent: matlab-style tree
%           - tree.descr: name of the node
%       tubes:
%           - .src: source node of the tube
%           - .dst: destination node of the tube
%           - .srcIdx: ID of source node in the tree-array
%           - .dstIdx: ID of destination node in the tree-array
%       wires:
%           - .src: source node of wire
%           - .dst: destination node of wire
%           - .srcPort: name of the connected port at the source
%           - .dstPort: name of the connected port at the destination
%           - .id: unique ID of the wire
%           - .condRad: radius of the conductor of the wire
%           - .dielThickness: radius of the sheath of the wire
%           - .e_r: relative permittivity of the sheath of the wire
%           - .sigma: conductivity of the conductor
%           optional:
%           - .sigma_d: conductivity of sheath
%           - .stranded: boolean indicating wether it is a stranded wire
%           - .strandRad: radius of one strand
%           - .numStrand: total number of strands in the wire
%           - .numOuter: number of strands constituting the outer ring
%           - .gnd: boolean indicating wether this is a reference wire
%       gndPlane: boolean indicating wether a ground plane is to be added
%       gndProp: properties of ground plane, if one should be added
%           - .gndSigma: conductivity of the ground plane
%           - .gndDist: distance of the ground plane to the conductors
%
% OUTPUT tree
%           -.ports: names of the ports this junction has
%           -.numPorts: number of ports at each junction
%        tubes
%           -.srcConn: connection array at the source junction
%           -.dstConn: connection array at the destination junction
%           -.numWires: number of wires in that tube
%           -.wireIdx: IDs of wires in that tube
%           -.gndPlane: boolean indicating wether a ground plane is present
%           -.NCond: number of conductors (=numWires + double(gndPlane))
%           optional:
%           -.gndDist: distance between gnd plane and conductors (only if
%           gndPlane = true);
%           -.gndSigma: conductivity of ground plane (only if
%           gndPlane=true)
%           -.gndIdx: index of reference wire (only if gndPlane=false)
%
%
%
% Explanation of src/dstConn: y = (src|dst)Conn(x) means that the wire no x
% in the tube connects to port no y at the (src|dst) of that tube.
%
% NOTE: a wire may have a field 'gnd', indicating wether this should be the
% reference wire. Only one wire per tube may have 'gnd' set to true, and if
% gndPlane is true then the 'gnd' field will be ignored and the ground
% plane will be used as reference.
%
% This alogrithm relies on the existance of the fields 'srcIdx' and
% 'dstIdx' in 'tubes' and the existance of the field 'parent' in 'tree'.
% Therefore it should be called after createTree.m

%printf-debugging?
DEBUG = false;
% DEBUG = true;

%check for ground plane
if (exist('gndPlane', 'var'))
    if (gndPlane)
        if (~isstruct(gndProp))
            error('gndProp assumed to be a struct');
        end
        if (~isfield(gndProp, 'gndDist'))
            error('gndProp must contain gndDist field');
        end
        gndDist = gndProp.gndDist;
        if (~isfield(gndProp, 'gndSigma'))
            error('gndProp must contain gndSigma field');
        end
        gndSigma = gndProp.gndSigma;
        
        if (isfield(wires, 'gnd'))
            if (0 ~= sum([wires.gnd]))
                disp('assignWires: WARNING: got gnd plane and gnd wires.');
                disp('assignWires: Ignoring gnd wires');
            end
        end
    end
else
    gndPlane = false;
end

%some santiy checks for already given fields
if (isfield(tubes, 'srcConn') || isfield(tubes, 'dstConn'))
    disp('assignWires: connection vector already given?');
end

%some sanity checks for required fields in tubes
if (~isfield(tubes, 'srcIdx') || ~isfield(tubes, 'dstIdx'))
    error('srcIdx and dstIdx fields in "tubes" required.');
end
if ( DEBUG && ( ~isfield(tubes, 'src') || ~isfield(tubes,'dst') ) )
    error('if DEBUG is enabled, src and dst fields in "tubes" required');
end

%some sanity checks for required fields in tree
if (~isfield(tree, 'parent'))
    error('tree structure not found. Field "parent" in "tree" expected');
end
if (~isfield(tree, 'descr'))
    error('tree must contain field "descr"');
end

%some sanity checks for required fields in wires
if ( ~isfield(wires, 'src') || ~isfield(wires, 'dst') )
    error('wires needs to contain fields "src" and "dst"');
end
if ( ~isfield(wires, 'srcPort') || ~isfield(wires, 'dstPort') )
    error('wires needs to contain fields "srcPort" and "dstPort"');
end
if ( ~isfield(wires, 'id')  )
    error('wires needs to contain fields "id"');
end
if ( ~isfield(wires, 'condRad') || ~isfield(wires, 'dielThickness') )
    error('wires needs to contain fields "condRad" and "dielThickness"');
end
if ( ~isfield(wires, 'e_r') || ~isfield(wires, 'sigma') )
    error('wires needs to contain fields "e_r" and "sigma"');
end
%if there is no ground plane, then there must be a gnd field in
%wires
if (~gndPlane && ~isfield(wires, 'gnd'))
    error('no ground plane and no reference wire specified');
end

%number of tubes
numTubes = length(tubes);
%number of wires per tube
numWires = zeros(1, numTubes);
%how many wires in this tube are already conncted?
connIdx = ones(1, numTubes);

%ids of ground wire in a tube, if there is one. zero means no gnd wire
gndIdx = zeros(1, numTubes);

%every jct has an infinite number of ports to which wires may be connected.
% There are two cases:
% - the wire "transits" the jct: the jct is neither src nor dst of the
% wire. Therefore we need to use an unused port to put the wire through (so
% this is only a "virtual" port, physically the wire just runs through)
% - the wire ends / begins at that jct: we need a uniquely identified port
% (ideally one that maps to the port ID given in 'wires') where all wires
% that physically connect to that port get connected
%
% To accomplish this, we reserve the number of ports this junction has (can
% be seen from the srcPort field of the struct that have that jct as src) in
% the lower region. So if that jct has 3 ports, we reserve the lower 3 IDs
% for those wires that are physically connected to that wire. All other
% wires will be connected to ports with higher IDs. To know where wires are
% connected and where not, we introduce an array: portsUsed. It initially
% is set to the number of reserved ports, and everytime a wire is put
% through that jct we use that variable and increase it by one. If a wire
% starts/ends at that jct we don''t change portsUsed, as that wire goes
% into the lower ID region that was already accounted for by the start
% value. The exact ID to which a wire that starts/ends at a junction is
% determined again by the srcPort field, so a wire that connects to the
% x-th port of the srcPort field, it is connected to the x-th port. In
% order to know which index in srcConn/dstConn to use, we keep track of the
% number of wires in a tube that we already connceted, in the connIdx
% array.

%%%%%
%Sometimes we will only need wires that are not ground wires
%%%%%
if (~gndPlane)
    nonGndWires = wires(~[wires.gnd]);
else
    nonGndWires = wires;
end

%%%%%
%Determine the number of reserved ports each junction has.
%%%%%
numJcts = length(tree);
portsReserved = zeros(1, numJcts);
for k = 1:numJcts
    %look for all wire that have this jct as port
    srcWireIdx = strcmp(tree(k).descr, {nonGndWires.src});
    dstWireIdx = strcmp(tree(k).descr, {nonGndWires.dst});
    
    tree(k).ports = unique(union( ...
        {nonGndWires(srcWireIdx).srcPort}, ...
        {nonGndWires(dstWireIdx).dstPort}));
    
    if (DEBUG)
        disp(['assignWires: ports at junction ' tree(k).descr ': ']);
        disp(tree(k).ports);
    end
    
    portsReserved(k) = length(tree(k).ports);
end

if (DEBUG)
    disp('assignWires: initially reserved ports at junctions:');
    disp({tree.descr});
    disp(portsReserved);
end

portsUsed = portsReserved;

%%%%%%
%assign wires to tube
%%%%%%
%we will look at every wire, determine a path through the tubes for that
%wire, add that wire to all the tubes and then finally connect the wire to
%the ports of the junctions in that path.
for k = 1:length(wires)
    
    %source and destination nodes name for the current wire
    src = wires(k).src;
    dst = wires(k).dst;
    
    if (DEBUG)
        disp('*****************');
        disp('assignWires: looking for path through tubes for wire: ');
        disp([wires(k).id ': ' src ' -> ' dst]);
        disp('*****************');
    end
    
    %indices of source and destination nodes in the tree
    srcIdx = strcmp(src, {tree.descr});
    dstIdx = strcmp(dst, {tree.descr});
    
    %sanity check if the node was found in the tube tree
    if (1 < sum(srcIdx))
        error(['more than one node with name ' src ' found?']);
    end
    if (1 < sum(dstIdx))
        error(['more than one node with name ' dst ' found?']);
    end
    
    %find a path of tube nodes through the tube tree from the wires source
    %to the wires destination.
    [sPath ~] = findPath([tree.parent], find(srcIdx), find(dstIdx));
    
    %it might happen that a wire has the same source and destination
    if (isempty(sPath))
        continue;
    end
    
    if (DEBUG)
        disp('assignWires: tubes path found: ');
        disp({tree(sPath).descr});
    end
    
    %%%%%%%%%%%%%%%%%
    %add all wire's properties to the tubes
    %%%%%%%%%%%%%%%%%
    for l = 2:length(sPath)
        %tube leading to this node
        [~, tubeIdx] = findEdge(sPath(l-1), sPath(l), tubes);
        
        %update number of wires in tube
        numWires(tubeIdx) = numWires(tubeIdx) + 1;
                
        nW = numWires(tubeIdx);
        %update properties of tube
        tubes(tubeIdx).wireIdx{nW} = wires(k).id;
        tubes(tubeIdx).condRad(nW) = wires(k).condRad;
        tubes(tubeIdx).dielThickness(nW) = wires(k).dielThickness;
        tubes(tubeIdx).e_r(nW) = wires(k).e_r;
        tubes(tubeIdx).sigma(nW) = wires(k).sigma;
        tubes(tubeIdx).sigma_d(nW) = 0;
        if (isfield(wires, 'sigma_d'))
            tubes(tubeIdx).sigma_d(nW) = wires(k).sigma_d;
        end
        tubes(tubeIdx).stranded = false;
        if (isfield(wires, 'stranded'))
            tubes(tubeIdx).stranded(nW) = wires(k).stranded;
        end
        tubes(tubeIdx).strandRad(nW) = 0;
        if (isfield(wires, 'strandRad'))
            if (~isempty(wires(k).strandRad))
                tubes(tubeIdx).strandRad(nW) = wires(k).strandRad;
            end
        end
        tubes(tubeIdx).numOuter(nW) = 0;
        if (isfield(wires, 'numOuter'))
            if (~isempty(wires(k).numOuter))
                tubes(tubeIdx).numOuter(nW) = wires(k).numOuter;
            end
        end
        tubes(tubeIdx).numStrand(nW) = 0;
        if (isfield(wires, 'numStrand'))
            if (~isempty(wires(k).numStrand))
                tubes(tubeIdx).numStrand(nW) = wires(k).numStrand;
            end
        end
        
        if (~gndPlane)
            if (wires(k).gnd)
                if (0 ~= gndIdx(tubeIdx))
                    disp(['tube ' num2str(tubeIdx) ...
                        '(' tubes(tubeIdx).src '->' tubes(tubeIdx).dst ')'...
                        ' already has ground wire with id ' ...
                        tubes(tubeIdx).wireIdx{gndIdx(tubeIdx)} ...
                        ', but wire with id ' wires(k).id ...
                        ' is supposed to be ground wire, too']);
                    error('only one ground wire per tube supported');
                end
                if (DEBUG)
                    disp(['assignWires: tube ' ...
                        num2str(tubeIdx) '(' tubes(tubeIdx).src '->' ...
                        tubes(tubeIdx).dst ') gets ground wire with ID ' ...
                        wires(k).id '.']);
                end
                gndIdx(tubeIdx) = numWires(tubeIdx);
            end
        end
        
    end
    
    
    %if the wire is a ground wire, don't do the connection arrays. It is
    %assumed that there is exactly one ground wire per tube, and that all
    %ground wires at a junction are connected with each other.
    
    if (~gndPlane)
        if (wires(k).gnd)
            if (DEBUG)
                disp(['assignWires: wire ' num2str(wires(k).id) ...
                    ' (' wires(k).src '->' wires(k).dst ') ' ...
                    'is the ground wire in tube ' ...
                    num2str(tubeIdx) '(' tubes(tubeIdx).src '->' ...
                    tubes(tubeIdx).dst ').']);
            end
            
            %ground wire isn't considered in the connection arrays
            continue;
        end
    end
           
    %%%%%%%%%%%%%%%%%%
    %connection arrays
    %%%%%%%%%%%%%%%%%%
    
    %%%%%
    %connect the wire to the first junction. We do this seperatedly, as the
    %first junction will be the one where the wire has a reserved port
    %%%%%
    
    %available ports at source
    wireSrcJctPorts = tree(sPath(1)).ports;
    %port wire connects to at source
    wireSrcPort = wires(k).srcPort;
    %index of the port the wire connects to in the list of available ports
    wireSrcPortIdx = strcmp(wireSrcPort, wireSrcJctPorts);
    if (1 ~= sum(wireSrcPortIdx))
        error(['wire supposed to connect to port ' wireSrcPort ...
            ', couldn''t be found in list of ports of junction ' src]);
    end
    %tube that is the first part of the path from wires src to wires dst
    [srcTube srcTubeIdx] = findEdge(sPath(1), sPath(2), tubes);
    %update connection arrays
    if (strcmp(src, srcTube.src)) %wires source is also tubes source
        tubes(srcTubeIdx).srcConn(connIdx(srcTubeIdx)) = find(wireSrcPortIdx);
    elseif (strcmp(src, srcTube.dst)) %wires source is the tubes destination
        tubes(srcTubeIdx).dstConn(connIdx(srcTubeIdx)) = find(wireSrcPortIdx);
    else
        error(['first tube in path from ' srcTube.src ' to ' srcTube.dst ...
            ', but wires source is ' src]);
    end
    
    if (DEBUG)
        disp(['assignWires: tube ' num2str(srcTubeIdx) ...
            '(' srcTube.src '->' srcTube.dst '): wire no ' ...
            num2str(connIdx(srcTubeIdx)) ' connected to port no ' ...
            num2str(find(wireSrcPortIdx)) '(' wireSrcPort ...
            ') at node ' src]);
    end
    
    %what we did until now: connect the wire to the first junction. No need
    %to update connIdx of the first tube, as we still need to connect the 
    %cable to the other side of the first tube.
    
    %%%%%
    %connect wire to junctions in the middle of the path
    %%%%%
    %in every step, we connect the wire to one node -> we consider two
    %tubes, one going to that node and one coming from that node. The
    %connIdx is only incremented for the tube going to that node.
    for l = 2:length(sPath)-1
        
        %name of this node
        jct = tree(sPath(l)).descr;
        
        %tube leading to this node
        [inTube inTubeIdx] = findEdge(sPath(l-1), sPath(l), tubes);
        
        %connect to the next free port
        if (strcmp(jct, inTube.src))
            tubes(inTubeIdx).srcConn(connIdx(inTubeIdx)) = portsUsed(sPath(l)) +1;
        elseif (strcmp(jct, inTube.dst))
            tubes(inTubeIdx).dstConn(connIdx(inTubeIdx)) = portsUsed(sPath(l)) +1;
        else
            error(['tube in path from ' inTube.src ' to ' inTube.dst ...
                ', but node that the tube is supposed to connect to is ' jct]);
        end
        
        if (DEBUG)
            disp(['assignWires: tube ' num2str(inTubeIdx) ...
                '(' inTube.src '->' inTube.dst '): wire no ' ...
                num2str(connIdx(inTubeIdx)) ' connected to port no ' ...
                num2str(portsUsed(sPath(l))+1) '(unnamed) at node ' jct]);
        end
        
        %the tube leading to this tube is completely connected
        connIdx(inTubeIdx) = connIdx(inTubeIdx) + 1;
               
        
        %tube going to the next node
        [outTube outTubeIdx] = findEdge(sPath(l), sPath(l+1), tubes);
        
        if (strcmp(jct, outTube.src))
            tubes(outTubeIdx).srcConn(connIdx(outTubeIdx)) = portsUsed(sPath(l)) +1;
        elseif (strcmp(jct, outTube.dst))
            tubes(outTubeIdx).dstConn(connIdx(outTubeIdx)) = portsUsed(sPath(l)) +1;
        else
            error(['tube in path from ' outTube.src ' to ' outTube.dst ...
                ', but node that the tube is supposed to connect to is ' jct]);
        end
        
        if (DEBUG)
            disp(['assignWires: tube ' num2str(outTubeIdx) ...
                '(' outTube.src '->' outTube.dst '): wire no ' ...
                num2str(connIdx(outTubeIdx)) ' connected to port no ' ...
                num2str(portsUsed(sPath(l))+1) '(unnamed) at node ' jct]);
        end
        
        %the outgoing tube is not connected at its other end -> no need to
        %update its connIdx.
        
        %two wires (in and outgoing) are connected to the current port at
        %the current jct, mark as used
        portsUsed(sPath(l)) = portsUsed(sPath(l)) + 1;
    end
    
    %what we did until now: connect the wire to all the tubes
    %except the last one. In the last tube, the wire is connected to the
    %node that is not the wires destination, but not yet to the wires
    %destination.
    
    %%%%%
    %connect the wire to the last junction
    %%%%%
   
    %available ports at last junction
    wireDstJctPorts = tree(sPath(end)).ports;
    %port wire connects to at source
    wireDstPort = wires(k).dstPort;
    %index of the port the wire connects to in the list of available ports
    wireDstPortIdx = strcmp(wireDstPort, wireDstJctPorts);
    if (1 ~= sum(wireDstPortIdx))
        error(['wire supposed to connect to port ' wireDstPort ...
            ', couldn''t be found in list of ports of junction ' src]);
    end
    %tube that is the last part of the path to wires dst
    [dstTube dstTubeIdx] = findEdge(sPath(end-1), sPath(end), tubes);
    %update connection arrays
    if (strcmp(dst, dstTube.src)) %wires destination is tubes source
        tubes(dstTubeIdx).srcConn(connIdx(dstTubeIdx)) = find(wireDstPortIdx);
    elseif (strcmp(dst, dstTube.dst)) %wires destination is tubes destination
        tubes(dstTubeIdx).dstConn(connIdx(dstTubeIdx)) = find(wireDstPortIdx);
    else
        error(['last tube in path from ' dstTube.src ' to ' dstTube.dst ...
            ', but wires destination is ' dst]);
    end
    
    if (DEBUG)
        disp(['assignWires: tube ' num2str(dstTubeIdx) ...
            '(' dstTube.src '->' dstTube.dst '): wire no ' ...
            num2str(connIdx(dstTubeIdx)) ' connected to port no ' ...
            num2str(find(wireDstPortIdx)) '(' wireDstPort ...
            ') at node ' dst]);
    end
    
    %the last tube is now connected at both ends -> increment connIdx
    connIdx(dstTubeIdx) = connIdx(dstTubeIdx) + 1;
    
end

%assign number of ports used to every node
tmp = num2cell(portsUsed);
[tree(:).numPorts] = deal(tmp{:});

%%%%%%%
%set the number of wires in every tube, do some sanity checks
%%%%%%%
for k = 1:numTubes
    tubes(k).numWires = numWires(k);
    tubes(k).NCond = numWires(k);
    
    if (gndPlane)
        tubes(k).gndPlane = true;
        tubes(k).gndSigma = gndSigma;
        tubes(k).gndDist = gndDist;
        tubes(k).NCond = tubes(k).NCond + 1;
        if (isfield(gndProp, 'gndWidth'))
            tubes(k).gndWidth = gndProp.gndWidth;
        end
        if (isfield(gndProp, 'gndHeight'))
            tubes(k).gndHeight = gndProp.gndHeight;
        end
    else
        tubes(k).gndPlane = false;
        if (0 == gndIdx(k))
            disp(['there was a ground wire in some tubes, but not in tube ' ...
                num2str(k) ' (' tubes(k).src '->' tubes(k).dst ')']);
            error(['no gnd plane, and not all tubes have a gnd wire']);
        end
        tubes(k).gndIdx = gndIdx(k);
    end
    
    
    %check if the connection arrays have the right number of entries
    if ( (connIdx(k)-1 ~= length(tubes(k).srcConn)) || ...
            (connIdx(k)-1 ~= length(tubes(k).dstConn)) )
        if (DEBUG)
            disp(['tube no ' num2str(k) ...
                '(' tubes(k).src ' -> ' tubes(k).dst '):']);
            disp(['connIdx: ' num2str(connIdx(k)) ...
                ', number of connection entries: ' ...
                num2str(length(tubes(k).srcConn)) '/' ...
                num2str(length(tubes(k).srcConn))]);
        end
        error('number of connection entries mismatch');
    end
    
    %check that there is no zero in the connection array
    zeroIdx = find( (tubes(k).srcConn == 0) | (tubes(k).dstConn == 0), 1 );
    if (~isempty(zeroIdx))
        error(['connection ID for wire ' num2str(zeroIdx) ' not set']);
    end
    
    %check that the connection arrays don't connect to ports that aren't
    %there
    [scm scmi] = max(tubes(k).srcConn);
    if (scm > portsUsed(tubes(k).srcIdx))
        if (DEBUG)
            disp(['reduceTree: node ' tubes(k).src ' has ' ...
                num2str(numPorts(tubes(k).srcIdx)) ...
                ' ports, but wire ' num2str(scmi) ...
                ' is supposed to connect to port ' num2str(scm)]);
        end
        error(['faulty connection array for tube ' num2str(k)]);
    end
    
    [dcm dcmi] = max(tubes(k).dstConn);
    if (dcm > portsUsed(tubes(k).dstIdx))
        if (DEBUG)
            disp(['reduceTree: node ' tubes(k).dst ' has ' ...
                num2str(numPorts(tubes(k).dstIdx)) ...
                ' ports, but wire ' num2str(dcmi) ...
                ' is supposed to connect to port ' num2str(dcm)]);
        end
        error(['faulty connection array for tube ' num2str(k)]);
    end
    
end

end
