function hash = sdbmTubeHash(NCond, condRad, dielThickness, e_r, condPos)
%SDBMTUBEHASH generates a very simple hash of a tubes wire config
%
% SYNOPSIS hash = sdbmTubeHash(NCond, condRad, dielThickness, e_r, condPos)
%
% INPUT NCond: number of conductors in the tube
%       condRad: radius of the conductors in the tube
%       dielThickness: sheats thickness
%       e_r: realtive permittivity of the sheats
%       condPos: optional, (NCondx2), conductors position.
% OUTPUT hash: hash of that tube
%
% NOTE: it is a really very very simple hashing function
% Also note: the ground plane is currently only included in the hash
% because NCond is one plus if a ground plane is there.

numWire = length(condRad);

%sanity checks
if (1 == length(dielThickness))
    dielThickness = dielThickness * ones(1, numWire);
end
if (numWire ~= length(dielThickness))
    error('size mismatch between condRad and dielThickness');
end
if (1 == length(e_r))
    e_r = e_r * ones(1, numWire);
end
if (numWire ~= length(e_r))
    error('size mismatch between condRad and e_r');
end

if (exist('condPos', 'var'))
    if (numWire ~= size(condPos, 1))
        error('size mismatch between condRad and condPos');
    end
end

%strings that will be hashed
str = cell(1, numWire);

%create strings for every wire
for k = 1:numWire
    if (exist('condPos', 'var'))
        str{k} = [num2str(condPos(k, :)) '_' num2str(condRad(k)) '_' num2str(dielThickness(k)) '_' num2str(e_r(k))];
    else
        str{k} = [num2str(condRad(k)) '_' num2str(dielThickness(k)) '_' num2str(e_r(k))];
    end
end

%sort the strings, so that it does not matter in which order the wires were
%specified
sortStr = sort(str);

%concatenate strings
toHash = [sortStr{:}];

%sdbm algorithm, http://www.eecs.harvard.edu/margo/papers/usenix91/paper.ps
hash = zeros(size(toHash,1),1);
for i=1:size(str,2),
    hash = mod(hash * 65599 + toHash(:,i), 2^32-1);
end

%prefix number of conductors
hash = [num2str(NCond) '_' num2str(hash)];

end