function H_SIMO = MTLPULTF(f, nodes, Y_L, Y_TX, edges, PUL, tx, rx, txWireId)
%MTLPULTF computes the SIMO-TF for a setup, given the PUL parametres
%
% SYNOPSIS H_SIMO = MTLPULTF(f, nodes, Y_L, Y_TX, edges, PUL, tx, rx)
%
% INPUT f: array with frequencies at which to evaluate the TF
%       nodes: cell array with strings containing the nodes' names
%       Y_L: cell array of load admittance matrices
%       Y_TX: transmitter admittance matrix
%       edges: struct describing the edges connecting the nodes
%       PUL: cell array containing the PUL parameters for the edges
%       tx: name of the transmitter
%       rx: name of the receiver
%       txWireId: id of the wire used at the transmitter for transmission
%
% OUTPUT H_SIMO: SIMO transfer function from the first wire of tx to any
% wire at rx
%
%

%createTree expects nodes to be a cell array of strings (the names) of
%the nodes. edges needs to contain only src and dst fields, both the
%name of a junction (strings) . root will be the root of the
%tree, and also must be the name of a junction (string)
root = nodes{1}; %arbitrary... may be any of the nodes
[tree edges] = createTree(nodes, edges, root);
numPorts = num2cell(genNumPorts(tree, edges));
[tree(:).numPorts] = deal(numPorts{:});


%if you want a visual representation of your tree, uncomment:
% treeplotWText(tree);


%we call it an edge when we are referring to the representation of the
%setup with a tree, and a tube when we are referring to the fact that its
%not just an edge, but it has wires in it and those wires have electrical
%properties. In other words, edges and tubes are different representations
%of the same physical thing.

nNodes = length(nodes);
nTubes = length(edges);

%sanity checks
if (length(Y_L) ~= nNodes)
    error('load admittance matrices need to be specified for every node');
end
if (length(PUL) ~= nTubes)
    error('PUL parameters need to be specified for every tube');
end

%look for tx and rx in nodes
txID = find(strcmp(tx, nodes), 1);
rxID = find(strcmp(rx, nodes), 1);

if (isempty(txID))
    error('tx not found in nodes');
end
if (isempty(rxID))
    error('rx not found in nodes');
end

%electrical properties of each tube
tubesElProp = struct('f', {f}, 'Y', cell(1, nTubes), 'Z', cell(1, nTubes));

%get impedance and admittance for all tubes
for k = 1:nTubes
    [Y Z] = PULYZ(f, PUL{k});
    tubesElProp(k).Y = Y;
    tubesElProp(k).Z = Z;
end


[Y_IN H_MIMO] = reduceTree(tree, edges, tubesElProp, txID, rxID, Y_L);

R_S = 0; %source resistance
disp(['Source resistance: ' num2str(R_S)])


H_SIMO = singleLineTF(H_MIMO, Y_TX, Y_IN, R_S, txWireId);
