%Consider the following setup: Transmitter (TX) and receiver (RX) are
%connected by four wires. A branch cable with two wires is connected at a
%branch point (BP). The destination of the branch cable is called BR. We
%denote TX, RX, BP and BE "nodes", and following the tree-terminonolgy, we
%define the section between two nodes as an edge. Using this, we can
%identify three edges in the setup: edge one from TX to BP, edge two from BP
%to RX and edge three from BP to BE. The number of wires in one edge and
%the wires properties always remain constant (Therefore, one edge can be
%described by one set of PUL paramters).
%
%                   BP
%   ____1_____________._________3____
%   ____2____________.__________1____
%TX ____3___________.___________2____ RX
%   ____r__________._\__________r____
%                   \ \
%                    \ \
%                     r 1
%                      \ \
%                       \ \
%                        BE
%
% To describe how the single wires of the tubes are connected at
% the branch point (and anywhere else in more complicated setups),
% we introduce the abstract concept of a "port": A port
% is a entity at a node of the tree. A node may have unlimited ports. One
% wire connects to exactly one port at a given node (therefore, one wire
% connects to a total of two ports: one port at the wires source, and one
% port at the wires destination) . Several wires may connect to the same
% port. Wires that connect to the same port are assumed to be connected at
% that node.
% A connection array is defined as follows: y = conn(x) means that wire
% number x of the edge is connected to port y at  the edges source node.
% Each tube has two connection arrays: one for the tube's source, and one
% for its destination.
%
% PLEASE BE AWARE: Each tube needs to 
% contain exactly one reference wire. Furthermore it is assumed that all
% reference wires are always connected. In the above case, we assume that
% wire "r" is the reference wire. THE REFERENCE WIRE IS NOT ACCOUNTED FOR
% IN THE CONNECTION ARRAYS!
%
% The connection arrays for the above example therefore could read:
% Tube 1 TX->BP: srcConn = [1 2 3]
%                dstConn = [1 2 3]
% Tube 2 BP->RX: srcConn = [2 3 1]
%                dstConn = [1 2 3];
% Tube 3 BP->BR: srcConn = [3];
%                dstConn = [1];
%
% The srcConn array for tube 1, and the dstConn arrays for tube 2 and 3 are
% only important when we define the load admittance matrices at those
% points, as they must match what we define here. (i.e, if we use say 3 =
% conn(1), then the load admittance matrix must contain the load connected
% to wire no 1 as the third entry.
%
% Notice how the actual number of the port at the node in the middle is
% irrelevant, as long as all wires that are connected to it have the same
% number. It is advisable however to use as small numbers as possible,
% because the load admittance matrix for a node is (nxnxnf), where n is the
% number of ports and nf is the number of frequencies, and the number of
% ports is determined by taking the maximum id of any connection array at
% that node.

addpath('pul'); %closed form solutions in there
addpath('../common'); %common files with HarnessTF

%frequencies
f = 1e5:1e5:35e6;
nFreq = length(f);

%names for all nodes (in this case, we have a transmitter, a receiver, a
%branch endpoint, and the junction where the branch meets the main
%transmission line
nodes = {'TX', 'RX', 'BE', 'BP'};

%every node has a load admittance matrix
Y_L = cell(1, 4);
Y_L{1} = zeros(3,3,nFreq); %load at the transmitter
Y_L{2} = repmat(diag([1/50 1/50 1/50]), [1 1 nFreq]); %load at the receiver
Y_L{3} = repmat(1/50, [1 1 nFreq]); %load at the branch endpoint
Y_L{4} = zeros(3,3,nFreq); %load at the junction

%this matrix describes the load at the nodes at the transmitter that are
%not used for signal transmission. Dimension is therefore (NCond-2 x
%NCond-2 x nFreq)
Y_TX = repmat(diag([1/50 1/50]), [1 1 nFreq]);


%next we describe the edges between the nodes
%edges connecting the trees.
% - src: source node of the edge
% - dst: destination node of the edge
% - len: length of the edge
% - numWires: number of wires running in that edge
% - (src,dst)Conn: connection arrays at (source,destination)

edges = struct( ...
    'src', {'TX', 'BP', 'BP'}, ...
    'dst', {'BP', 'RX', 'BE' }, ...
    'len', {  10,   10,   10 }, ...
    'numWires', {4, 4, 2}, ...
    'srcConn', { [1 2 3], [2 3 1], [2] }, ...
    'dstConn', { [1 2 3], [1 2 3], [1] } ...
    );

%every edge is described by its per unit length parameters
PUL = cell(1, 3);
PUL{1} = load('pulData/ribbon_4cond.mat');
PUL{2} = load('pulData/symm_4cond.mat');
PUL{3} = @flat_two; %PUL parameters can also be specified as function handles
%The interface for a PUL function is as follows:
% [R C L G] = fun(freq), where freq is a scalar (frequency in Hz).

%name of the transmitter
tx = 'TX'; % = nodes{1}
%name of the receiver
rx = 'RX'; % = nodes{2}

%ID of the wire used for transmitting
txWireId = 1;

%generates the SIMO transfer function, assuming that the reference and the
%first wire were used for transmission at the transmitter side
H_SIMO = MTLPULTF(f, nodes, Y_L, Y_TX, edges, PUL, tx, rx, txWireId);

%H_SIMO contains all transfer functions from the first wire on the
%transmitter side to all wires on the receiver side

%the third entry is the direct transfer function, as the first wire of the
%edge from the transmitter to the branch point is connected to the wire
%labeled "3" on the edge from the branch point to the receiver.
H_d = 20*log10(abs(H_SIMO(3, :))); %direct transfer function
printFun(f/1e6, H_d, 'auto', 'example tf', '', 'onebranch_TF');
