function [Y, Z] = PULYZ(f, PUL)
%PULYZ creates Y and Z from PUL parameters
%
% SYNOPSIS [Y Z] = PULYZ(f, PUL)
%
% INPUT f: frequencies at which to compute Y and Z
%       PUL: struct with the following fields:
%               - f: frequency vector
%               - C: capacitance matrix (nxn) or (nxnxnf)
%               - L: inductance matrix (nxn) or (nxnxnf)
%               - R: resistance matrix (nxn) or (nxnxnf)
%               - G: conductance matrix (nxn) or (nxnxnf)
%               where n is the number of conductors minus one and nf is the
%               number of frequencies for which the matrices are specified
%            or a function handle: [R C L G] = PUL(f)
%
% OUTPUT Y: admittance matrix
%        Z: impedance matrix
%
%
% NOTICE: this code assumes that the third dimension of all parameters
%is the frequency, if they have 3 dimensions

%if a function handle was given, we assume it is a function that returns R
%C L and G for a given frequency.

numFreq = length(f);

if (isa(PUL, 'function_handle'))
    [r, ~, ~, ~] = PUL(f(1));
    numEle = size(r, 1);
    R = zeros(numEle, numEle, numFreq);
    C = zeros(size(R));
    L = zeros(size(R));
    G = zeros(size(R));
    for k = 1:length(f)
        [r, c, l, g] = PUL(f(k));
        R(:,:,k) = r;
        C(:,:,k) = c;
        L(:,:,k) = l;
        G(:,:,k) = g;
    end

else
    
    if (~isfield(PUL, 'f'))
        error('PULYZ:input', 'frequencies f are expected in PUL');
    end
    
    %check if PUL parameters are given for all frequencies specified in f
    if (~isempty(setdiff(f, PUL.f)))
        error('PULYZ:PULf', 'frequencies with unknown PUL parameters requested');
    end
    
    fIdx = ismember(PUL.f, f);
    
    if (isfield(PUL, 'Y') && isfield(PUL, 'Z'))
        Y = PUL.Y(:,:,fIdx);
        Z = PUL.Z(:,:,fIdx);
        
        %nothing to be done here
        return;
    end
    
    %sanity checks
    if (~isfield(PUL, 'C'))
        error('Capacitance matrix ''C'' expected');
    else
        C = PUL.C;
    end
    
    if (~isfield(PUL, 'L'))
        error('Inductance matrix ''L'' expected');
    else
        L = PUL.L;
    end
    if (~isfield(PUL, 'G'))
        error('Conductance matrix ''G'' expected');
    else
        G = PUL.G;
    end
    if (~isfield(PUL, 'R'))
        error('Resistance matrix ''R'' expected');
    else
        R = PUL.R;
    end
    
    %"repair" matrices that are frequency independent
    if (1 == size(R, 3))
        R = repmat(R, [1 1 length(fIdx)]);
    else
        R = R(:,:,fIdx);
    end
    
    if (1 == size(C, 3))
        C = repmat(C, [1 1 length(fIdx)]);
    else
        C = C(:,:,fIdx);
    end
    
    if (1 == size(L, 3))
        L = repmat(L, [1 1 length(fIdx)]);
    else
        L = L(:,:,fIdx);
    end
    
    if (1 == size(G, 3))
        G = repmat(G, [1 1 length(fIdx)]);
    else
        G = G(:,:,fIdx);
    end
end

numEle = size(C, 1);

Y = zeros([numEle, numEle, numFreq]);
Z = zeros(size(Y));

for k = 1:numFreq
    Z(:,:,k) = R(:,:,k) + 1i*2*pi*f(k)*L(:,:,k);
    Y(:,:,k) = G(:,:,k) + 1i*2*pi*f(k)*C(:,:,k);
end

end