function [tree, tubes] = createTree(junctions, tubes, root)
%CREATETREE: creates a matlab-tree from a set of junctions and tubes
%
% SYNOPSIS: [tree, tubes] = createTree(junctions, tubes, root)
%
% INPUT junctions: cell array with the names of the junctions
%       tubes: array of structs with the following fields:
%           - src: junction from which this tube originates
%           - dst: junction that is this tube's destination
%           - may contain other fields that will be copied
%       root: name of the junction that should be root of the tree
% OUTPUT    tree:
%               .parent: matlab-style struct array that represents the tree
%               .descr: name of the node
%           tubes: same as tubes, but with srcIdx and dstIdx fields added.
%
% tree:
% is a struct with two fields: 'descr' and 'parent'.
% y = tree(x).parent means that node with ID x (with name junctions{x} ==
% tree(x).descr) has node with ID y (with name junctions{y} == 
% tree(y).descr) as parent in the tree.
% If y is 0, then the node with ID x is the root node.
%
% the output tubes:
% tubes is a copy of the input called 'tubes', but two fields are added:
% srcIdx and dstIdx. As the junctions are assigned IDs, the IDs are copied
% into the 'tubes' struct so that one can find the edge to two nodes
% without knowing their names. This way, the names of the junctions can be
% dropped from further processing.
%
% Loops in the tree will not be detected but simply "cut open" at any
% location convenient, but the algorithm should never find itself in an
% endless loop (hopefully).
%

if (~exist('root', 'var'))
    root = junctions{1};
end

numNodes = length(junctions);
numEdges = length(tubes);


%sanity check
if (isfield(tubes, 'srcIdx'))
    error('arguemnt "tubes" already has a field called srcIdx?');
end
if (isfield(tubes, 'dstIdx'))
    error('argument "tubes" already has a field called dstIdx?');
end

%the new tree, matlab syntax
tree = struct('parent', -1, 'descr', junctions);

%check if the specified root node is a junction (as it should be)
[rootIsMember rootIdx] = ismember(root, junctions);

if (~rootIsMember)
    error(['root node "' root '" not found in "junctions"']);
end

%root node in matlab syntax is indicated by a 0
tree(rootIdx).parent = 0;

%housekeeping arrays of visited (position in the tree is fixed) and 
%pending (can be reached from a visited node with one hop) nodes
visitedNodes = {};
pendingNodes = {root};


while(~isempty(pendingNodes))
    
    %take one node from queue
    currentNode = pendingNodes{1};
    currentNodeIdx = find(strcmp(currentNode, junctions));
    if (isempty(currentNodeIdx))
        error(['createTree: node "' currentNode '" not found in "junctions"']);
    end
    pendingNodes = setdiff(pendingNodes, currentNode);
    
    %tubes that have connection to the current node, undirected graph
    for edgeIdx = find(strcmp(currentNode, {tubes.src}));
        %get other end of edge
        dstNode = tubes(edgeIdx).dst;
        %check if we visited that one alread
        if (ismember(dstNode, visitedNodes))
            continue;
        end
        
        %assign current node as parent of other end of edge
        dstNodeIdx = strcmp(dstNode, junctions);
        if (0 == sum(dstNodeIdx))
            disp(['createTree: edge ' num2str(edgeIdx) ': ' ...
                'src: ' tubes(edgeIdx).src ', dst: ' tubes(edgeIdx).dst]);
            error(['edge ' num2str(edgeIdx) ' claims to have nonexistant node "' dstNode '" as destination.']);
        end
        tree(dstNodeIdx).parent = currentNodeIdx;
        
        %add current edge to tubes
        tubes(edgeIdx).srcIdx = currentNodeIdx;
        tubes(edgeIdx).dstIdx = find(dstNodeIdx);
        
        %add destination node to list of pending nodes
        pendingNodes = union(pendingNodes, dstNode);
    end
    
    %above code again for other direction
    for edgeIdx = find(strcmp(currentNode, {tubes.dst}));
        srcNode = tubes(edgeIdx).src;
        if (ismember(srcNode, visitedNodes))
            continue;
        end
        srcNodeIdx = strcmp(srcNode, junctions);
        if (0 == sum(srcNodeIdx))
            disp(['createTree: edge ' num2str(edgeIdx) ': ' ...
                'src: ' tubes(edgeIdx).src ', dst: ' tubes(edgeIdx).dst]);
            disp(['createTree: edge ' num2str(edgeIdx) ': ' ...
                'src: ' tubes(edgeIdx).src ', dst: ' tubes(edgeIdx).dst]);
            error(['edge ' num2str(edgeIdx) ' claims to have nonexistant node "' srcNode '" as destination.']);
        end
        tree(srcNodeIdx).parent = currentNodeIdx;
        
        tubes(edgeIdx).srcIdx = find(srcNodeIdx);
        tubes(edgeIdx).dstIdx = currentNodeIdx;
        
        pendingNodes = union(pendingNodes, srcNode);
    end
    
    %add current node to list of visited nodes
    visitedNodes = union(visitedNodes, currentNode);
    
    %just to be sure...
    pendingNodes = setdiff(pendingNodes, visitedNodes);

end

%check if we got all nodes
unvisitedNodes = setdiff(junctions, visitedNodes);
if (~isempty(unvisitedNodes))
    disp('createTree: WARNING: some nodes got left unvisited:');
    disp(unvisitedNodes);
end

%check if we got all tubes
if (isfield(tubes, 'dstIdx') && isfield(tubes, 'srcIdx'))
    for k = 1:numEdges
        if (isempty(tubes(k).dstIdx) || isempty(tubes(k).srcIdx))
            disp(['createTree: WARNING: dstIdx or srcIdx of edge ' num2str(k) ...
                ' from node ' tubes(k).src ' to node ' tubes(k).dst ...
                ' left unset!']);
            disp('createTree: this probably means that this edge was not needed to create the tree');
        end
    end
else
    disp('createTree: srcIdx and dstIdx left unset?');
    error('fields didn''t get created as they should have, check your input');
end

%check if there is only one root node
if (1 ~= sum([tree.parent] == 0))
    disp('createTree: tree hasn''t exactly one root node:');
    disp({tree(0 == [tree.parent]).descr});
    error('tree creation failed');
end

%check if any nodes were left unassigned
if (0 ~= sum([tree.parent] == -1))
    disp('createTree: WARNING: some junctions left without a parent:');
    disp({tree(-1 == [tree.parent]).descr});
end