function [sPath prev] = findPath(nodes, startNode, endNode)
%FINDPATH computes the path from startNode to endNode in a tree
%
% SYNOPSIS [sPath prev] = findPath(nodes, startNode, endNode)
%
% INPUT nodes: array where y = nodes(x) means that y is the parent of x
%       startNode: index of the start point
%       endNode: index of the end point.
%
% OUTPUT sPath: array with node indices in the path
%        prev: new tree that uses startNode as root (y = prev(x) again
%        means that y is the parent of x)
%
% The algorithm assumes that the graph spanned by the nodes is a tree,
% i.e there is only one path from a node to another.
% The algorithm itself is a simplified version of the dijkstra algorithm.
%

if (startNode == endNode)
    disp(['findPath: startNode is equal to endNode (' ...
        num2str(startNode) ')!']);
    sPath = [];
    prev = [];
    return;
end

%get total number of nodes in the tree
numNodes = length(nodes);

%array that holds the indices of the predecessor to every node
prev = zeros(1,numNodes);

%array that holds inidces of notes not yet examined
unvisited = 1:numNodes;

%queue of nodes that have been seen (i.e can be reached by one hop from
%already visited nodes), but have not yet been visited
queue = startNode;

%as long as there are nodes that have been seen, but not visited, visit the
%next node
while (~isempty(queue))
    %get new node in queue
    nodeIdx = queue(1);
    %remove current node from queue
    queue = queue(2:end);
    %set current node as visited
    unvisited = setdiff(unvisited, nodeIdx);
    
    %get all unvisited neighbors
    
    %get         children             and parent          nodes
    newNodes = [ find(nodes==nodeIdx) nodes(nodeIdx) ];
    %remove nodes from 'newNodes' that have already been visited
    newNodes = intersect(newNodes, unvisited);
    %remove 0, as it is simply a special indicator that the node has no
    %parent (-> is the root node)
    newNodes = setdiff(newNodes, 0);
    
    %set previous node for all new nodes to the current node
    for k = 1:length(newNodes)
        prev(newNodes(k)) = nodeIdx;
    end
    
    %add unvisited new nodes to queue
    queue = [queue newNodes];
end

nodeIdx = endNode;
sPath = [];
%traverse graph backwards, writing down the path that we took
while(0 ~= prev(nodeIdx))
    sPath = [nodeIdx sPath];
    nodeIdx = prev(nodeIdx);
end

%check if we found a valid path
if (sPath(end) ~= endNode)
    error(['Could not find path from node ' num2str(startNode) ' to ' num2str(endNode)]);
end

%sanity check that there is only one root node in our new tree
if (1 ~= sum(prev == 0))
    disp('original tree');
    disp(nodes);
    disp('new tree');
    disp(prev);
    error('More than one root node in the new tree?');
end

%add the starting node to the path
sPath = [startNode sPath];
    
end