function Y_L = genRndLoad(settings, numRealPorts, f)
%GENRNDLOAD generates random load admittance matrix
%
% SYNOPSIS Y_L = genRndLoad(prop, numRealPorts, f)
% 
% INPUT settings    : .RMIN, RMAX, .IMIN, .IMAX, .DIFFLOADPROB
%       numRealPorts: number of real ports at the node
%       f           : frequencies
%
% OUTPUT Y_L: load admittance matrix (numRealPorts x numRealPorts x length(f))
%
%
% Note that the returned load admittance matrices are not frequency
% dependent (the load admittance matrix is the same for all frequencies)



%sanity checks
if (~isfield(settings, 'RMIN'))
    error('need RMIN');
end
if (~isfield(settings, 'RMAX'))
    error('need RMAX');
end
if (~isfield(settings, 'IMIN'))
    error('need IMIN');
end
if (~isfield(settings, 'IMAX'))
    error('need IMAX');
end
if (~isfield(settings, 'DIFFLOADPROB'))
    error('need DIFFLOADPROB');
end

%shortcuts
RMIN = settings.RMIN;
RMAX = settings.RMAX;
IMIN = settings.IMIN;
IMAX = settings.IMAX;
DIFFLOADPROB = settings.DIFFLOADPROB;

numFreq = length(f);

%load admittance
Y_L = zeros(numRealPorts, numRealPorts, numFreq);

%every port is assigned exactly one admittance
for m = 1:numRealPorts
    sign = randi([0 1])*2-1; %sign of the imaginary part
    y = 1/(randi([RMIN, RMAX]) + sign*1i*randi([IMIN, IMAX]));
    y = repmat(y, [1 1 numFreq]);
    Y_L(m, m, :) = Y_L(m, m, :) + y;
    
    %Evaluate if the load should be differential (between two wires)
    if (randi(100) <= DIFFLOADPROB)
        port = randi(numRealPorts);
        Y_L(port, port, :) = Y_L(port, port, :) + y;
        Y_L(m, port, :) = Y_L(m, port, :) - y;
        Y_L(port, m, :) = Y_L(port, m, :) - y;
    end
end