function [lConn pConn] = getConn(edge, leaf, parent)
%GETCONN returns connection vectors from edge for (leaf,parent)-pair
%
% SYNOPSIS [lConn pConn] = getConn(edge, leaf, parent)
%
% INPUT edge: struct that represents edge
%       leaf: ID of leaf node (must be either source or destination of
%       edge)
%       parent: ID of parent node (must be the other)
%
% OUTPUT lConn: connection vector for the leaf
%        pConn: connection vector for the parent
%
% This algorithm checks if the leaf is source or destination, and returns
% source/destination connection vectors mapped correctly to lConn and
% pConn.
%
% edge must contain the following fields:
%  - srcIdx, dstIdx: indices of source and destination nodes
%  - srcConn, dstConn: source and destination connection matrices

if (~isfield(edge, 'srcConn') || ~isfield(edge, 'dstConn'))
    error('edge must contain fields (src,dst)Conn');
end
if (~isfield(edge, 'srcIdx') || ~isfield(edge, 'dstIdx'))
    error('edge must contain fields (src,dst)Idx');
end

if ~( ((leaf == edge.srcIdx) && (parent == edge.dstIdx)) || ...
      ((leaf == edge.dstIdx) && (parent == edge.srcIdx)) )
  disp(['leaf: ' num2str(leaf) ', parent: ' num2str(parent)]);
  disp(edge);
  error('leaf and parent ID aren''t equal to srcIdx/dstIdx | dstIdx/srcIdx of edge!');
end

if (edge.srcIdx == leaf)
    lConn = edge.srcConn;
    pConn = edge.dstConn;
else
    lConn = edge.dstConn;
    pConn = edge.srcConn;
end

%%%%
%sanity checks
%%%%

if (~isempty(setdiff(unique(lConn), lConn)) || ...
    ~isempty(setdiff(unique(pConn), pConn)) )
    disp('conn array repeated itself:');
    disp(edge);
    error('connection array error')
end


if (isfield(edge, 'NCond'))
    NCond = edge.NCond;
    %check connection array length
    if (length(lConn) ~= NCond-1)
        disp(['edge from ' edge.src ' to ' edge.dst ' has ' ...
            num2str(NCond) ' conductors, but lConn has ' ...
            num2str(length(lConn)) ' entries']);
        error('connection array dimension mismatch');
    end
    
    %check connection array length
    if (length(pConn) ~= NCond-1)
        disp(['edge from ' edge.src ' to ' edge.dst ' has ' ...
            num2str(NCond) ' conductors, but lConn has ' ...
            num2str(length(pConn)) ' entries']);
        error('connection array dimension mismatch');
    end
end

end