function printFun(X, Y, lim, descr, figTitle, fname)
%PRINTFUN plots data nicely
%
% SYNOPSIS printFun(X, Y, lim, descr, figTitle, fname)
%
% INPUT X: cell array for x-axis
%       Y: cell array for y-axis
%       lim: limits for x- and y-axis (or string 'auto')
%       descr: what will appear as legend
%       figTitle: figure title
%       fname: file name, if the plot is to be saved
%
% OUTPUT --
%
% if fname is given, the font size will be increased and the figure will be
% saved to fname.eps and fname.fig

%sanity checks
if (~exist('fname', 'var'))
    PRINT = false;
elseif (isempty(fname))
    PRINT = false;
else
    PRINT = true;
end

if (~exist('figTitle', 'var'))
    figTitle = '';
end


%if no cell is given for Y or descr, assume only one plot

if (~iscell(Y))
    y = Y;
    Y = cell(1,1);
    Y{1} = y;
    clear('y');
end

if (~iscell(descr))
    descrr = descr;
    descr = cell(1,1);
    descr{1} = descrr;
    clear('descrr');
end
    
NUMMARKERS = 15; %number of markers per line
LINEWIDTH = 2; %width of line
NUMXTICK = 11;
NUMYTICK = 11;
FONTSIZE = 16; %font size
FONTSIZE_SMALL = 16; %font size for the legend
MARKERSIZE = 7; %size of markers
%LOC = 'NorthWest'
%LOC = 'NorthEast'
%LOC = 'SouthWest'
LOC = 'SouthEast'; %location of the legend
% LOC = 'BestOutside';


if (isempty(lim))
    lim = 'auto';
end
%sanity check for axis limits dimensions
if (~ischar(lim) && length(lim) ~= 4)
    error('expected lim to be array [xmin xmax ymin ymax]');
end

%sanity checks for number of plots in figure
numFun = length(descr);
if (length(Y) ~= numFun)
    error('expected same number of cells in Y as in descr');
end

if (~iscell(X))
    x = X;
    X = cell(1, numFun);
    for k = 1:numFun
        X{k} = x;
    end
    clear x;
end

if (length(X) ~= numFun)
    error('expected same number of cells in X as in descr');
end

%sanity checks for number of samples in each plot
for k = 1:numFun
    if (length(Y{k}) ~= length(X{k}))
        error(['plot no ' num2str(k) ' does not have the same number '...
            'of sampling points as the x axis vector.']);
    end
end

%markers for different plot to increase readability on b/w plots
markers = {'+','o','*','.','x','s','d','^','v','>','<','p','h'};

%just for (my) convenience, limit number of plots per figure
if (numFun > length(markers))
    disp(['Number of plots: ' num2str(numFun)]);
    disp(['maximum of supported plots: ' num2str(length(markers))]);
    error('too much plots for this function. note that descr has to be a cell array!');
end

%create figure and adjust axes settings
gcf = figure;
gca = axes();
set(gcf, 'CurrentAxes', gca);

if (~ischar(lim) && (4 == length(lim)))
    set(gca, 'XLimMode', 'manual');
    set(gca, 'YLimMode', 'manual');
    set(gca, 'XLim', lim(1:2));
    set(gca, 'YLim', lim(3:4));
    set(gca, 'XTick', linspace(lim(1), lim(2), NUMXTICK));
    set(gca, 'YTick', linspace(lim(3), lim(4), NUMYTICK));
end

hold on;
grid on;
%set title
title(figTitle);
%get colormap
cols = colormap(lines(numFun));

%get sampling for the markers
STEP = floor(cellfun('length', X)./NUMMARKERS);
OFF = floor(STEP./(numFun+1)); %offset so markers don't overlap

%plot dummy points so that the legend matches (with markers and lines)
for k = 1:numFun
    plot(0,0, 'Color', cols(k, :), 'Marker', markers{k}, 'MarkerSize', MARKERSIZE, 'linewidth', LINEWIDTH, 'Visible', 'Off');
end

%plot the evenly spaced markers, without lines
for k = 1:numFun
    y = Y{k};
    x = X{k};
    plot(x(1+(k-1)*OFF(k):STEP(k):end), y(1+(k-1)*OFF(k):STEP(k):end), 'Color', cols(k, :), 'Marker', markers{k}, 'MarkerSize', MARKERSIZE, 'linewidth', LINEWIDTH, 'linestyle', 'none');
end
%plot the lines, without markers
for k = 1:numFun
    plot(X{k}, Y{k}, 'Color', cols(k, :), 'linewidth', LINEWIDTH);
end

%create legend
lh = legend(descr, 'Location', LOC);

%
%latex pretty print: increased fontsize and less ticks so that things fit
%if figure is scaled down

if (PRINT)
    set(lh, 'FontSize', FONTSIZE_SMALL);
    xlabel('xlabel', 'fontsize', FONTSIZE); %increase fontsize so that there is enough place for the latex replacements
    ylabel('ylabel', 'fontsize', FONTSIZE);
    title(figTitle, 'fontsize', FONTSIZE);
    
    %the ticks are set so that the grid is nice
    yTick = get(gca, 'YTick');
    yTickLabel = num2cell(yTick);
    yTickLabel(2:2:end) = {' '};
    set(gca, 'YTickLabel', yTickLabel);
    xTick = get(gca, 'XTick');
    xTickLabel = num2cell(xTick);
    xTickLabel(2:2:end) = {' '};
    set(gca, 'XTickLabel', xTickLabel);
    set(gca, 'FontSize', FONTSIZE);
    
    %output, save as figure if we want to change something later
    saveas(gcf, [fname '.fig']);
    %and as .eps, if everythings ok
    print(gcf, '-depsc2', [fname '.eps']);
end

end