function [Y_IN H_TOT] = reduceTree(jcts, edges, elProp, txId, rxId, Y_Ls)
%REDUCETREE computes transfer function from one node in a tree to another
%
% SYNOPSIS [Y_IN H_TOT] = reduceTree(jcts, edges, elProp, txId, rxId, Y_Ls)
%
% INPUT jcts: .parent: matlab-style tree of nodes
%             .numPorts: number of ports available at the nodes
%       edges: struct array that describes the edges that connect the nodes
%             .src, dst: source and destination of edge
%             .scrIdx, dstIdx: indices of source and destination
%             .srcConn, dstConn: connection vector at source and
%             destination
%             .len: length of that edge
%       elProp: struct array with electrical properties of the edges
%             .f: array with all frequencies at which parameters are given
%             .Y, Z: characteristic admittance and impedance matrix
%       txId: id of the transmitter in 'nodes'
%       rxId: id of the receiver in 'nodes'
%       Y_Ls: cell array containing load admittance matrices of nodes
%
% OUTPUT Y_IN: load admittance matrix carried back to tx
%        H_TOT: overall MIMO-TF from tx to rx
%
% a matlab style tree of nodes means that if y = nodes(x), y is parent of
% node x.


%printf-debugging?
DEBUG           = true; %general
DEBUGY          = DEBUG & false; %Y for single wires at specifiy nodes
DEBUGTF         = DEBUG & false; %single wire TF (needs to be specified)
DEBUGTREE       = DEBUG & false; %for the tree-handling


%We can chose single wires, nodes, and ports here for printf-debugging

% if (DEBUGTF)
%     %for C346<->C347
%     TXW = '5010B';
%     RXW = '5010B';
%     TXWPort = 'A1';
%     RXWPort = 'B5';

%     %for C153<->C251
%     TXW = '8000C'; %wire used for transmitting
%     RXW = '8000B'; %wire used for receiving
%     TXWPort = '7'; %port at which transmitting wire is connected to tx
%     RXWPort = '7'; %port at which receiving wire is connected to rx

%     %for C310<->C053
%     TXW = '2053G';
%     RXW = '2053DA';
%     TXWPort = 'F4';
%     RXWPort = 'E';
% end

% if (DEBUGY)
%     %for C310<->C053
%     NODEWATCHLIST = {'SPL052', 'JCT11', 'JCT12'};
%     WIREWATCHLIST = {'2053G', '2053DA'};
    
%     %for C153<->C251
%     NODEWATCHLIST = {'SPL078', 'JCT16', 'JCT17', 'SPL014', 'SPL037', ...
%         'SPL038', 'SPL039', 'JCT21'};
%     WIREWATCHLIST = {'8000B', '8000C'};
% end

%sanity checks
if ( (1 ~= length(txId)) || (1 ~= length(rxId)) )
    error('expected scalar txId and rxId');
end

if (~isfield(jcts, 'parent'))
    error('expected jcts to have field parent');
end
if (~isfield(jcts, 'numPorts'))
    error('expected jcts to have field numPorts');
end

nodes = [jcts.parent];
numPorts = [jcts.numPorts];
numNodes = length(nodes);
if ( (txId > numNodes) || (rxId > numNodes) )
    error('txId or rxId not in range of valid node indices');
end

if (isempty(txId) || isempty(rxId))
    error('must provide rx and tx ID');
end

if (~iscell(Y_Ls))
    error('"Y_Ls" must be cell array');
end

if (length(Y_Ls) ~= numNodes)
    error('number of load admittance matrices doesn''t match number of nodes');
end

if (length(elProp) ~= length(edges))
    error('expected elecrical properties of every tube');
end

if (~isfield(elProp, 'f') || ~isfield(elProp, 'Y') || ~isfield(elProp, 'Z'))
    error('for every edge, "f", "Y", and "Z" must be specified');
end

if (~isfield(edges, 'srcIdx') || ~isfield(edges, 'dstIdx'))
    error('ids of src and dst nodes must be specified (call createTree first)');
end
if (~isfield(edges, 'len'))
    error('length of edges must be specified');
end

%find backbone
[backbone, tree] = findPath(nodes, txId, rxId);

if (DEBUGTREE)
    disp('reduceTree: found a backbone: ');
    disp(backbone);
    disp('reduceTree: tree: ');
    disp(tree);
end

%get indices of all leaves that are not part of the backbone
%find(tree~=-1): indices of all nodes still in the tree
%setdiff(ans, tree): indices of all nodes that aren't a parent of any node
%                   -> leaves
%setdiff(ans, backbone): no nodes that are part of the backbone
%
%-> 'leaves' contains indices of leaves that are not part of the backbone
leaves = setdiff(setdiff(find(tree~=-1), tree), backbone);

if (DEBUGTREE)
    disp('reduceTree: leaves: ');
    disp(leaves);
end

while(~isempty(leaves))
    
    %remove one leaf
    leaf = leaves(end);
    leaves = leaves(1:end-1);
    
    parent = tree(leaf);
    
    %find edge that connets the two nodes
    [edge, edgeID] = findEdge(leaf, parent, edges);
    [lConn, pConn] = getConn(edge, leaf, parent);
    edgeElProp = elProp(edgeID);
    
    numFreq = length(edgeElProp.f);
    %check if load admittance matrix at current node has the right size
    [dim1, dim2, dim3] = size(Y_Ls{leaf});
    if (~isequal([dim1 dim2 dim3], [numPorts(leaf), numPorts(leaf), numFreq]))
        disp(['reduceTree: edge''s parameters specified for ' ...
            num2str(numFreq) ' frequencies, and node ' ...
            num2str(leaf) ' has ' ...
            num2str(numPorts(leaf)) ' ports,' ...
            ' but size of load admittance matrix is: ']);
        disp(size(Y_Ls{leaf}));
        error('load admittance matrix dimension mismatch');
    end
    
    %downconvert node's Y_L to match number of wires on edge
    Y_LEdge = node2EdgeLoad(Y_Ls{leaf}, lConn);
%     Y_LEdge = Y_Ls{leaf}(lConn, lConn, :);
    

    if (DEBUGY)
        leafName = jcts(leaf).descr;
        %check if we are at a node on the watchlist
        if (ismember(leafName, NODEWATCHLIST))
            disp('load at leaf:');
            disp(['current node: "' leafName '" (' num2str(leaf) ')']);
            disp(['current edge: (' edge.src ') -> (' edge.dst '), ' ...
                num2str(edge.len)]);
            %get all wires on the watchlist that are in the current edge
            wires = ismember(edge.wireIdx, WIREWATCHLIST);
            wirePorts = lConn(wires);

            %display the load at the watchlist wires
            disp(['load at ports ' num2str(wirePorts) ' of leaf ']);
            disp(Y_Ls{leaf}(wirePorts, wirePorts, 1));

            disp(['load at wires ' num2str(find(wires)) ' at leaf: ']);
            disp(Y_LEdge(wires, wires, 1));
        end
    end

    %check some dimensions
    if (~isequal(size(edgeElProp.Y), size(edgeElProp.Z)))
        disp('reduceTree: edge: ');
        disp(edge);
        error('size of Y and Z don''t match for edge!');
    end
    if (~isequal(size(Y_LEdge), size(edgeElProp.Y)))
        disp('reduceTree: Dimensions of load admittance matrix and Y don''t match: ');
        disp('reduceTree: size of load admittance matrix: ');
        disp(size(Y_LEdge));
        disp('reduceTree: size of Y:');
        disp(size(edgeElProp.Y));
        disp('reduceTree: original size of load admittance matrix: ');
        disp(size(Y_Ls{leaf}));
        disp('reduceTree: leaf connection array: ');
        disp(lConn);
        disp('reduceTree: edge: ');
        disp(edge);
        error('dimension mismatch');
    end
    if (length(edgeElProp.f) ~= size(edgeElProp.Y, 3))
        disp('reduceTree: Y not specified for number of frequencies in ''f''');
        disp(['reduceTree: Y is specified for ' num2str(size(edgeElProp.Y, 3)) ...
            ' frequencies, ''f'' contains ' num2str(length(edgeElProp.f)) ' values']);
        disp(['reduceTree: note: it is assumed that Y is a '...
            '(Ncond-1xNcond-1xnumFreq) matrix']);
        error('dimension mismatch');
    end

    %carry back load to upstream (-> towards backbone) node
    [Y_R, ~] = wireTF(edgeElProp.f, edgeElProp.Y, edgeElProp.Z, edge.len, Y_LEdge);
    
    %upconvert Y_R to match number of ports on parent    
    Y_RPort = zeros(numPorts(parent), numPorts(parent), numFreq);
    Y_RPort(pConn, pConn, :) = Y_R;
    
    if (~isequal(size(Y_Ls{parent}), size(Y_RPort)))
        disp(['reduceTree: load carried back from node ' num2str(leaf) ...
            ' to node ' num2str(parent) ' and load admittance matrix '...
            'at that node have different sizes: ']);
        disp('reduceTree: parent:');
        disp(size(Y_Ls{parent}));
        disp('reduceTree: carried back from leaf:');
        disp(size(Y_RPort));
        error('dimension mismatch');
    end
    
    Y_Ls{parent} = Y_Ls{parent} + Y_RPort;
    
    if (DEBUGY)
        leafName = jcts(leaf).descr;
        if (ismember(leafName, NODEWATCHLIST))
            disp('load at parent: ');
            disp(['current node: "' leafName '" (' num2str(leaf) ')']);
            disp(['current edge: (' edge.src ') -> (' edge.dst '), ' ...
                num2str(edge.len)]);
            %get all wires on the watchlist that are in the current edge
            wires = ismember(edge.wireIdx, WIREWATCHLIST);
            wirePorts = lConn(wires);
            disp(['load at wires: ' num2str(find(wires)) ' at parent: ']);
            disp(Y_R(wires, wires, 1));
            disp(['additional load at ports: ' num2str(wirePorts) ...
                ' of parent: ']);
            disp(Y_RPort(wirePorts, wirePorts, 1));
            disp('accumulated load at ports of parent: ');
            disp(Y_Ls{parent}(wirePorts, wirePorts, 1));
        end
    end
    
    if (DEBUGTREE)
        disp('reduceTree: removing the following leaf from the tree: ');
        disp(leaf);
    end

    tree(leaf) = -1; %detach current leaf from tree
    
    %conditions that the parent of the current leaf is another leaf after
    %the current leaf is removed:
    % - the current leaf is not the root (which shouldn't happen as the
    %       root should be part of the backbone)
    % - the parent of the current leaf is not itself part of the backbone
    % - the parent of the current leaf is not yet a leaf (shouldn't happen
    %       either because that would mean it was in there before it was a
    %       leaf)
    % - the parent of the current node does not have any child nodes
    if ( ismember(parent, backbone) ) %member of bb?
        if (DEBUGTREE)
            disp(['reduceTree: parent node ' num2str(parent) ...
                ' member of backbone, not adding']);
        end
    elseif ( isequal(0, parent) ) %leaf was root node?
        if (DEBUGTREE)
            disp(['reduceTree: current leaf ' num2str(leaf) ...
                ' is the root node? Doh!']);
        end
    elseif ( ismember(parent, tree) )%parent has another child?
        if (DEBUGTREE)
            disp(['reduceTree: parent node ' num2str(parent) ...
                ' has at least one other child (' ...
                num2str(find(tree==parent, 1, 'first')) ...
                '), not adding.']);
        end
    else
        leaves = [leaves parent];
    end
    
    if (DEBUGTREE)
        disp('reduceTree: new set of leaves:');
        disp(leaves);
    end
end

if (DEBUGTREE)
    disp('reduceTree: no more nodes that aren''t part of the backbone');
end

%now we have a tree with no branches that has txId as its root and rxId as
%its leaf.

if (DEBUGTF)
    txWPortId = find(strcmp(jcts(txId).ports, TXWPort));
    rxWPortId = find(strcmp(jcts(rxId).ports, RXWPort));
    
    if (isempty(txWPortId))
        error(['DEBUGTF: could not find port "' TXWPort ... 
            '" whose tf should be printed']);
    end
    if (isempty(rxWPortId))
        error(['DEBUGTF: could not find port "' RXWPort ...
            '" at receiver whose tf should be printed']);
    end

    disp(['wire "' TXW '" originates at port no ' num2str(txWPortId) ...
        ' of node "' jcts(txId).descr '"']);
    disp(['wire "' RXW '" originates at port no ' num2str(rxWPortId) ...
        ' of node "' jcts(rxId).descr '"']);
end

clear('Y_LEdge', 'Y_R', 'Y_RPort', 'edge', 'edgeElProp', 'edgeID', ...
    'lConn', 'leaf', 'leafName', 'leaves', 'pConn', 'parent')

%TODO: tree <-> nodes

%We have a completely reduced tree now, with one or more edges concatenated
% that lead from tx to rx

%we start at the end node
currNode = rxId;
numFreq = length(elProp(1).f);
H_TOT = repmat(eye(numPorts(currNode)), [1 1 numFreq]);

%-> as long as we aren't at the tx node
while(currNode ~= txId)
    %first, the load at this node is considered, then the line to its
    %parent node.
    parent = tree(currNode);
    [edge, edgeID] = findEdge(currNode, parent, edges);
    [lConn, pConn] = getConn(edge, currNode, parent);
    edgeElProp = elProp(edgeID);
    numFreq = length(edgeElProp.f);
    
    
    if (DEBUGTF)

        disp(['Current edge: (' edge.src ') -> (' edge.dst ')']);
        
        %check if any of the TXW/RXW is on the current edge
        tfWIdx = find(strcmp(edge.wireIdx, TXW));
        if (isempty(tfWIdx))
            tfWIdx = find(strcmp(edge.wireIdx, RXW));
            tfW = RXW;
        else
            tfW = TXW;
        end
        
        if (isempty(tfWIdx))
            disp(['neither "' TXW '" nor "' RXW '" found on curr. edge']);
        else
        
            disp(['current wire: "' tfW '"']);

            leavePortName = 'unnamed';
            arrivePortName = 'unnamed';
            if (length(jcts(currNode).ports) >= lConn(tfWIdx))
                leavePortName = jcts(currNode).ports{lConn(tfWIdx)};
            end
            if (length(jcts(parent).ports) >= pConn(tfWIdx))
                arrivePortName = jcts(parent).ports{pConn(tfWIdx)};
            end

            disp(['TF wire is no ' num2str(tfWIdx)]);
            disp(['TF wire leaves at node "' jcts(currNode).descr ...
                '", port ' num2str(lConn(tfWIdx)) '(' leavePortName ')']);
            disp(['TF wire arrives at node "' jcts(parent).descr '", port '...
                num2str(pConn(tfWIdx)) '(' arrivePortName ')']);
        end
    end
    
    %load admittance matrix for the current leaf:
    Y_LNode = Y_Ls{currNode};
    Y_LEdge = node2EdgeLoad(Y_LNode, lConn);
    
    %sanity check
    minNumLPorts = max(lConn);
    if (size(Y_LNode, 1) < minNumLPorts)
        disp('size of load admitance matrix:');
        disp(size(Y_L));
        disp('connection array:');
        disp(lConn);
        error('dimension mismatch');
    end
    
    %compute the carried back load and the TF for the current edge
    [Y_R, H_WIRE] = wireTF(edgeElProp.f, edgeElProp.Y, edgeElProp.Z, edge.len, Y_LEdge);
    
    %compute the TF from edge to the node
    H_LPORT = edge2NodeTF(H_WIRE, Y_LNode, lConn);
    
    H_PORTS = zeros(numPorts(currNode), numPorts(parent), numFreq);
    H_PORTS(:, pConn, :) = H_LPORT;
    
    if (DEBUGTF)
        disp(['abs TF for current cable for wire ' tfW ' (k = 1,end): ' ...
            num2str(abs(H_WIRE(tfWIdx, tfWIdx, 1))) ',' ...
            num2str(abs(H_WIRE(tfWIdx, tfWIdx, end)))]);
    end
    
    %get what is the input admittance to this edge, and the load admittance
    %of the parent node:
    Y_IN = Y_Ls{parent};
    
    %upconvert the edges carried back load admittance matrix to match the
    %number of ports on the parent node
    Y_RPort = zeros(size(Y_IN));
    Y_RPort(pConn, pConn, :) = Y_R;
    
    %The load admittance matrix at the parent node (input admittance
    %matrix for this edge) may already contain values from another edge.
    %-> add the carried back load admittance matrix
        
    Y_IN = Y_IN + Y_RPort;
    
    %the input admittance for this edge is equal to the load admittance
    %matrix for the upstream (-> towards the root) edge of the node
    Y_Ls{parent} = Y_IN;
    
    if (DEBUGY)
        leafName = jcts(currNode).descr;
        if (ismember(leafName, NODEWATCHLIST))
            disp(['current node: "' leafName '" (' num2str(currNode) ')']);
            disp(['current edge: (' edge.src ') -> (' edge.dst '), ' ...
                num2str(edge.len)]);
            %get all wires on the watchlist that are in the current edge
            wires = ismember(edge.wireIdx, WIREWATCHLIST);
            wireLeafPorts = lConn(wires);
            wireParentPorts = pConn(wires);
            disp(['load at ports: ' num2str(wireLeafPorts) ' of leaf: ']);
            disp(Y_LNode(wireLeafPorts, wireLeafPorts, 1));
            disp(['load at wires ' num2str(find(wires)) ' at the leaf: ']);
            disp(Y_LEdge(wires, wires, 1));
            disp('carried back load at wires: ');
            disp(Y_R(wires, wires, 1));
            disp(['carried back load ports ' num2str(wireParentPorts) ...
                ' of parent: ']);
            disp(Y_RPort(wireParentPorts, wireParentPorts, 1));
            disp('input admittance at parent ports: ');
            disp(Y_Ls{parent}(wireParentPorts, wireParentPorts, 1));
            disp('accumulated admittance at parent ports: ');
            disp(Y_IN(wireParentPorts, wireParentPorts, 1));
        end
    end
    
    [m, n, ~] = size(H_TOT);
    [p, q, ~] = size(H_PORTS);
    
    if (~isequal(n, p))
        disp('reduceTree: uh oh...');
        disp('reduceTree: size of total transfer function so far:');
        disp(size(H_TOT));
        disp(['reduceTree: size of wire transfer function from ' ...
            num2str(currNode) ' to ' num2str(parent) ': ']);
        disp(size(H_WIRE));
        disp('reduceTree: size of adapted tf from ports to ports: ');
        disp(size(H_PORTS));
        disp('reduceTree: connection arrays to current and parent node: ');
        disp(lConn);
        disp(pConn);
        error('dimension mismatch');
    end
    
    H_TOT2 = zeros(m, q, numFreq);
    
    %multiply transfer matrices
    for k = 1:numFreq
        H_TOT2(:,:,k) = H_TOT(:,:,k) * H_PORTS(:,:,k);
    end
    
    H_TOT = H_TOT2;
    
    if (DEBUGTF)
        disp(['abs total tf so far from node "' jcts(parent).descr ...
            '", port ' num2str(pConn(tfWIdx)) ' to node "' ...
            jcts(rxId).descr '", port ' num2str(rxWPortId) ...
            ' (' RXWPort '): '...
            num2str(abs(H_TOT(rxWPortId, pConn(tfWIdx), 1))) ]);
    end
    
    %advance one node
    currNode = parent;
end

end
