function H_SIMO = singleLineTF(H_MIMO, Y_TX, Y_I, R_S, inputId)
%SINGLELINETF computes the SIMO TF from the MIMO TF
%
% SYNOPSIS H_SISO = singleLineTF(H_MIMO, Y_TX, Y_I, R_S, inputId)
%
% INPUT H_MIMO: matrix with the MIMO TF
%       Y_TX: admittance matrix at the transmitter (tx)
%       Y_I: input admittance into the network as seen from tx
%       R_S: source resistance
%       inputId: number of the wire at the tx that was used for
%           transmitting
%
% OUTPUT H_SIMO: Transer functions, assuming that the inputId wire was the
% input wire
%


%sanity checks
if (~exist('inputId', 'var'))
    inputId = 1;
end

[nRx nTx numFreq] = size(H_MIMO);

[dim1, dim2, dim3] = size(Y_I);
if (~isequal([dim1, dim2, dim3], [nTx nTx numFreq]))
    disp('size of MIMO TF:');
    disp(size(H_MIMO));
    disp('size of input admittance matrix:');
    disp(size(Y_I));
    error('input dimension mismatch');
end

if (isequal(size(Y_TX), [nTx-1 nTx-1]))
    Y_TX = repmat(Y_TX, [1 1 numFreq]);
end
    

[dim1, dim2, dim3] = size(Y_TX);
if (~isequal([dim1 dim2 dim3], [nTx-1 nTx-1 numFreq]))
    %the tx admittance matrix should have one entry less than the MIMO TF,
    %as it is only for conductors not used for transmitting (-> 2 entries 
    %less than there are conductors)
    disp('size of MIMO TF:');
    disp(size(H_MIMO));
    disp('expected size:');
    disp([ num2str(nTx-1) ' ' num2str(nTx-1) ' ' num2str(numFreq)]);
    disp('size of tx admittance matrix:');
    disp(size(Y_TX));
    error('input dimension mismatch');
end

if (inputId > nTx)
    disp(['cannot use conductor ' num2str(inputId) ' as input: ']);
    error(['only ' num2str(nTx) ' conductors available as input']);
end


%for information on the algorithm, please refer to the thesis.
H_SIMO = zeros(nRx, numFreq);
ip = inputId;
nip = setdiff(1:nTx, inputId);
for k = 1:numFreq
    Y_ii = Y_I(ip, ip, k);
    Y_Pi = Y_I(nip, ip, k);
    Y_iP = Y_I(ip, nip, k);
    Y_PP = Y_I(nip, nip, k);
    
    gamma = zeros(nTx, 1);
    gamma(ip) = 1;
    gamma(nip) = -(Y_TX(:,:,k) + Y_PP)\Y_Pi;
    H_SIMO2 = H_MIMO(:,:,k) * gamma;
    
    y_1 = Y_ii + Y_iP * gamma(nip);
    
    H_SIMO(:, k) = ((1 + R_S*y_1) \ H_SIMO2);
end

end
