function [Y Z] = tubeYZ(f, tubeParam, GType, nodesPerUnit)
%TUBEYZ computes characteristic impedance and admittance for tube
%
% SYNOPSIS tubeElProp = tubeYZ(f, tubeParam, GType, nodesPerUnit)
%
% INPUT f: frequencies at which Y and Z should be computed
%       tubeParam: struct with description of tube
%           .e_r: (1xnumWires) relative permittivity of the wires
%           .sigma_d (optional if GType is fdep) (1xnumWires) conductivity
%           of the sheath
%           Furthermore, all fields required by your choice of genMatFP
%       GType: {fdep|findpeC|findepL}: define how G should be calculated
%       nodesPerUnit: number of nodes per unit of lenght for FDM
%
% OUTPUT: Y, Z: characteristic impedance and admittance for
%           tube

%directory where PUL parameters are
PULDATADIR = 'pulData/';

%printf-debugging?
DEBUG = false; %general
DEBUGHASH = false; %print hash, and if it was already computed or not
QUIET = true; %be quiet, do not print warnings

%function used to create input for mtlPUL. If you would want to simulate
%noncircular conductors, you could write your own function and use it here.
%The interface the function needs to implement is described in genMatCirc.m
genMatFP = @genMatCirc;

if (~exist('GType', 'var'))
    if (~QUIET)
        disp('tubeYZ: WARNING: GType not specified, assuming fdep');
    end
    GType = 'fdep';
end

if (~exist('nodesPerUnit', 'var'))
    if (~QUIET)
        disp('tubeYZ: WARNING: nodesPerUnit not specified, assuming 1000');
    end
    nodesPerUnit = 5000;
end
    
if (strcmp(GType, 'fdep'))
    if (DEBUG)
        disp('tubeYZ: using frequency dependent G');
    end
    if (isreal(tubeParam.e_r))
        if (~QUIET)
            disp(['tubeYZ: WARNING: frequency dependent G needs '...
                'complex e_r, but tube has real e_r']);
        end
    end
elseif (strncmp(GType, 'findep', 6))
    if (DEBUG)
        disp('tubeYZ: computing G from C or L');
    end
    if (~isfield(tubeParam, 'sigma_d'))
        error(['if GType is findepL, the electrical '...
            'conductivity of the wire must be specified '...
            'using sigma_d']);
    end
    if (~isreal(tubeParam.e_r))
        if (~QUIET)
            disp(['tubeYZ: WARNING: frequency independent C '...
                'obliterates complex e_r']);
        end
        tubeParam.e_r = real(tubeParam.e_r);
    end
else
    disp(['method to compute G: ' GType]);
    error('unknown method to compute G');
end

pulFname = '';
gotValidPUL = false;
if (isfield(tubeParam, 'hash'))
    %expect the PUL parameters to be saved at this location
    pulFname = [ PULDATADIR tubeParam.hash '.mat' ];
    
    if (exist(pulFname, 'file'))
        if (DEBUGHASH)
            disp(['tubeYZ: using precomputed PUL parameters for hash "'...
                tubeParam.hash '"']);
        end
        load(pulFname);
        gotValidPUL = true;
    else
        if (DEBUGHASH)
            disp(['tubeYZ: Computing Y and Z for tube with hash "' ...
                tubeParam.hash '"']);
        end
    end
else
    disp('it is advised to add a hash to tubeParam to identify equal tubes');
end

%if you were to use a different method to compute the PUL parameters, here
%is the place to do so
if (~gotValidPUL)

%     if (~isempty(pulFname))
%         save([PULDATADIR tubeParam.hash '_input.mat'], 'tubeParam', 'nodesPerUnit')
%     end

    gotValidResult = false;
    prevErr = '.';
    stepSize = ceil(nodesPerUnit/4);

    
    %two things can happen while trying to find the PUL parameters using
    %mtlPUL.m: Either a conductor is represented by too few pixels, or the
    %algorithm diverges. If the conductor is too small, the number of
    %pixels needs to be increased, and if the algorithm diverges, the
    %number of pixels needs to be decreased. Therefore we try to find an
    %optimal resolution here.
    while (~gotValidResult)
        [cIm E_R refC] = genMatFP(nodesPerUnit, tubeParam);
        disp([num2str(numel(E_R)) ' nodes.']);
        if (stepSize < .1)
            disp('Could not find a suitable resolution.');
            disp('Either the conductors were too small');
            disp('or the algorithm diverged');
            error('Problem is too large');
        end
        try
            [PUL.C PUL.L PUL.G] = mtlPUL(cIm, E_R, refC);
            gotValidResult = true;
        catch mtlPULerr
            disp('error catched');
            if (strcmp(mtlPULerr.identifier, 'mtlPUL:condTooSmall'))
%                 disp('error was: small');
                if (strcmp(prevErr, 'divergence'))
                    stepSize = stepSize/2;
                end
                nodesPerUnit = nodesPerUnit + stepSize;
                prevErr = 'condTooSmall';
            elseif (strcmp(mtlPULerr.identifier, 'mtlPUL:divergence'))
%                 disp('error was: divergence');
                nodesPerUnit = nodesPerUnit - stepSize;
                if (strcmp(prevErr, 'condTooSmall'))
                    stepSize = stepSize/2;
                end
                prevErr = 'divergence';
            else
                disp(mtlPULerr.message);
                error('unknown error');
            end
        end
%         disp([ 'nodesPerUnit: ' num2str(nodesPerUnit) ...
%             ', stepSize: ' num2str(stepSize) ]);
    end
    PUL.R = genResMat(f, tubeParam);
    PUL.f = f;

    MU0 = 4*pi*1e-7;
    EPS0 = 8.854187817e-12;

    
    %If G is to be computed from C or L, do so here.
    if (strcmp(GType, 'findepC'))
        if (DEBUG)
            disp(['tubeYZ: warning: using mean sigma_d and e_r to '...
                'compute G from C']);
        end
        sigma_d = mean(tubeParam.sigma_d);
        e_r = mean(tubeParam.e_r);
        PUL.G = PUL.C*(sigma_d/(EPS0*e_r));
    elseif(strcmp(GType, 'findepL'))
        if (DEBUG)
            disp(['tubeYZ: warning: using mean sigma_d to compute '...
                'G from L']);
        end
        sigma_d = mean(tubeParam.sigma_d);
        PUL.G = MU0*sigma_d*inv(PUL.L);
    end

    %if we have a valid filename, save the results
    if (~isempty(pulFname))
        save(pulFname, 'PUL', 'tubeParam');
    end
end

[Y Z] = PULYZ(f, PUL);

end
