function [Y_eq, H] = wireTF(f, Y, Z, len, Y_L)
%WIRETF computes carried back impedance and MIMO TF
%
% SYNOPSIS [Y_eq H] = wireTF(f, Y, Z, len, Y_L)
%
% INPUT f: frequencies at which the TF is to be computed. Y and Z must be
%           given for those frequencies.
%       Y: admittance matrix of the cable, must be (nxnxnf)
%       Z: impedance matrix of the cable, must be (nxnxnf)
%       len: length of the cable (m)
%       Y_L: the load admittance matrix. Must be (nxn) or (nxnxnf)
%
%           n is the number of conductors minus one, and nf is the number of
%           frequencies given in f.
%
%
% OUTPUT:   Y_eq: equivalent admittance matrix that can be seen at the input
%               of the cable
%           H: MIMO transfer function for the specified frequencies

%printf-debugging?
DEBUG = false;


%sanity checks
if (~isequal(size(Y), size(Z)))
    error('Y and Z must be of same size');
end
if ( (length(f) ~= size(Y, 3)) || (length(f) ~= size(Z, 3)) )
    error('third dimension of Y and Z must be frequency');
end


Y_eq = zeros(size(Y));
H = zeros(size(Y));

%check if Y_L is supplied frequency-independent
if (length(size(Y)) == 1+length(size(Y_L)))
    %this would mean Y_L is lacking one dimension and therefore was
    %specified as frequency independent.
    if (DEBUG)
        disp('wireTF: assuming Y_L is frequency independent');
    end
    Y_Ls = zeros(size(Y));
    for k = 1:length(f)
        Y_Ls(:,:,k) = Y_L;
    end
elseif(isequal(size(Y), size(Y_L)))
    Y_Ls = Y_L;
else
    disp('size of Y:');
    disp(size(Y));
    disp('size of Y_L:');
    disp(size(Y_L));
    error('size of Y and Y_L don''t match?');
end
    
    

%Fabio Versolatto, Andrea M. Tonello:
%An MTL Theory Approach for the Simulation of MIMO Power Line Communication
%Channels
for k = 1:length(f)
    
    [T, LAMBDA] = eig(Y(:,:,k)*Z(:,:,k));

    GAMMA = sqrt(LAMBDA);

    Y_C = ((T / GAMMA) / T) * Y(:,:,k);
%     Z_C = inv(Y(:,:,k)) * T * GAMMA * inv(T);
%     Y_C = inv(Z_C);

    RHO = T \ Y_C / (Y_Ls(:,:,k) + Y_C) * ((Y_Ls(:,:,k) - Y_C) / Y_C) * T;
%     RHO = inv(T) * Y_C * inv(Y_Ls(:,:,k) + Y_C) * (Y_Ls(:,:,k) - Y_C) * Z_C * T;
    gamma = diag(GAMMA);
    expp = diag(exp(gamma*len));  %e2
    expm = diag(exp(-gamma*len)); %e1
    
    Y_eq(:,:,k) = T * (expp + expm*RHO) / (expp - expm*RHO) / T * Y_C;
%     Y_eq(:,:,k) = T * (expp + expm*RHO) * inv(expp - expm*RHO) * inv(T) * Y_C;

    H(:,:,k) = Y_C \ T * (eye(size(RHO)) - RHO) / ...
        (expp - expm*RHO) / T * Y_C;
%     H(:,:,k) = Z_C * T * (eye(size(RHO)) - RHO) * inv(expp - expm*RHO) * inv(T) * Y_C;

end