function [ cIm E_R refC] = genMat(nodesMax, desc)
%GENMAT: creates input matrices for mtlCap from image and description
%
% SYNOPSIS  [ cIm E_R refC] = genMat(nodesMax, desc)
%
% INPUT nodesMax: approximate maximum number of nodes
%       desc: file name, where desc.mat is the description file, and
%       desc.png is the picture of the setup
%
% OUTPUT cIm: cell of matrices. Every matrix depicts one conductor, being
%             one where the conductor is present and zero where it isn't
%        E_R: matrix of the same size as the matrices in cIm, containing
%             relative permittivity of the region
%        refC: ID of reference conductor
%
% The description file must be a *.mat file or a struct,
% containing the following fields:
%   NCond   : number of conductors in the image
%   cond    : cell with rgb values of the conductors colors in the image
%   NDiel   : number of dielectrics in the image
%   diel    : cell with rgb values of the dielectrics colors in the image
%   e_r     : array with the relative permittivities of the dielectrics

PLOTINPUT = true;
scrSize = get(0, 'ScreenSize');
%check if display is available
if (PLOTINPUT && (isequal(scrSize(3:4), [1 1])))
    disp('genMat: Disabling input plotting because no screen could be found');
    PLOTINPUT = false;
end

descFname = [desc '.mat'];
imFname = [desc '.png'];

if (~exist(descFname, 'file'))
    error(['genMat: description file ' descFname ' not found']);
end
desc = load(descFname);

if (~exist(imFname, 'file'))
    error(['genMat: image file ' imFname ' not found']);
end

%some sanity checks
if (~isfield(desc, 'NCond'))
    error('descFname needs to contain variable NCond');
end
if (~isfield(desc, 'cond'))
    error('descFname needs to contain cell cond');
end

if (~isequal(length(desc.cond), desc.NCond))
    error('NCond must be equal to length of cell cond');
end

if (~isfield(desc, 'NDiel'))
    error('descFname needs to contain variable NDiel');
end
if (~isfield(desc, 'diel'))
    error('descFname needs to contain cell diel');
end
if (~isequal(length(desc.diel), desc.NDiel))
    error('NDiel must be equal to length of cell diel');
end

if (~isfield(desc, 'e_r'))
    error('descFname needs to contain array with dielectric permittivities');
end
if (size(desc.e_r,1) ~= size(desc.NDiel))
    error('first dimension of e_r must match number of dielectrica');
end

if (~isfield(desc, 'refC'))
    refC = 1;
    disp('genMat: WARNING: no reference conductor set. Using the first specified as reference.');
else
    refC = desc.refC;
end

if (~exist(imFname, 'file'))
    error(['Couldn''t find file ' imFname]);
end

%check image size
improp = imfinfo(imFname);
numPixels = improp.Width*improp.Height;

scale = sqrt(nodesMax/numPixels);

if (scale > 1)
    disp('Original image has less pixels than nodes required.');
    disp(['Computing with ' num2str(numPixels) ' nodes.']);
    im = imread(imFname);
else
    %load image of conductors and dielectrica
    %TODO: dielectrica and conductors in different images?
    im = imresize(imread(imFname), scale, 'nearest', 'Dither', false, 'Antialiasing', false);
end

im_column = reshape(im, [], 3);
im_colors = unique(im_column, 'rows');
im_colors = setdiff(im_colors, [255 255 255], 'rows');
desc_colors_all = reshape([desc.cond{:} desc.diel{:}], 3, [])';
desc_colors = unique(desc_colors_all, 'rows');
test = ismember(desc_colors_all, desc_colors, 'rows');
if (sum(test == 0) > 0)
    error('looks like two entities have the same color?');
end

colors1 = setdiff(im_colors, desc_colors, 'rows'); %returns values of im_colors that are not in desc_colors
if (~isempty(colors1))
    disp('Looks like there are colors in the image that aren''t in the description!');
    disp('colors:');
    for kk = 1:size(colors1, 1)
        disp(colors1(kk, :));
    end
    error('Color mismatch');
end

colors2 = setdiff(desc_colors, im_colors, 'rows'); %returns values of desc_colors that are not in im_colors
if (~isempty(colors2))
    disp('Looks like there are colors in the description that don''t exist in the image!');
    disp('colors:');
    for kk = 1:size(colors2, 1)
        disp(colors2(kk,:));
    end
    error('Please fix the files');
end

figure;
hold on;
plotDescr = cell(desc.NCond + desc.NDiel, 1);

cIm = cell(desc.NCond, 1);

for i = 1:desc.NCond
    col = desc.cond{i};
    if (length(col) ~= 3)
        disp(['Conductor ' num2str(i) ':']);
        error('expected the colors to be 3 entries r,g,b');
    end
    cIm{i} = logical( ...
        (im(:,:,1) == col(1)) .* ...
        (im(:,:,2) == col(2)) .* ...
        (im(:,:,3) == col(3)) ...
        );
    plot(1,1, 'Color', col/255, 'Visible', 'off');
    plotDescr{i} = ['Conductor ' num2str(i)];
end

E_R = ones(size(im,1), size(im,2));
for i = 1:desc.NDiel
    col = desc.diel{i};
    if (length(col) ~= 3)
        disp(['Dielectric ' num2str(i) ':']);
        error('expected the colors to be 3 entries r,g,b');
    end
    dIm = logical( ...
        (im(:,:,1) == col(1)) .* ...
        (im(:,:,2) == col(2)) .* ...
        (im(:,:,3) == col(3)) ...
        );
    E_R(dIm) = desc.e_r(i);
    plot(1,1, 'Color', col/255, 'Visible', 'off');
    plotDescr{i+desc.NCond} = ['\epsilon_r = ' num2str(desc.e_r(i)) ];
end

if (PLOTINPUT)
    
    legend(plotDescr);
    imshow(im);
    
end

end
