function [ cIm E_R refC] = genMatCirc(nodesPerUnit, desc)
%GENMATCIRC: generates input matrices for mtlPUL for ribbon cables
%
% SYNOMPSIS: [ cIm E_R refC] = genMatCirc(nodesPerUnit, baseFname, matAdd)
%
% INPUT nodesPerUnit: number of nodes per unit of distance
%       desc: struct or file name of the description file containing:
%           Mandatory:
%           -NCond: number of conductors (numWires + double(gndPlane)
%           -numWires: number of wires
%           -condPos: numWiresx2-matrix containing conductors centres position
%           -condRad: numWires-vector containing radii of conductors
%           -dielThickness: vector containing thickness of dielectrics
%           -e_r: vector containing relative permittivities of dielectrics
%           -xmin, xmax, ymin, ymax: scalars specifying problem region
%           Optional:
%           -dielIRad: vector containing inner radii of dielectrics. If not
%           given, dielIRad = condRad is assumed.
%           -NDiel: number of dielectrics
%           -dielPos: NDielx2-matrix containing dielectrics centres
%           -gndPlane: should a ground plane be added?
%           Mandatory, if gndPlane is true:
%           -gndWidth: widht of the ground plane to simulate
%           -gndHeight: height of the ground plane to simulate
%
% OUTPUT cIm: cell with logical matrices. Every conductor has its matrix
%			in one seperate cell, where 1s indicate the presence of the
%			conductor and 0 indicate that the conductor isn't there.
%			In other words, one 1/0-picture per conductor.
%        E_R: a matrix the same size of every cell in cIm that contains
%			the relative permittivity throughout the region of interest
%        refC: ID of the reference conductor
%

%shall the conductor configuration be plotted
PLOTINPUT = true;
%print debugging information?
DEBUG = false;

%check if display is available
scrSize = get(0, 'ScreenSize');
if (PLOTINPUT && (isequal(scrSize(3:4), [1 1])))
    disp('genMatCirc: Disabling input plotting because no screen could be found');
    PLOTINPUT = false;
end

if (~isstruct(desc))

    if (~exist(desc, 'file'))
        error(['File ' desc ' could not be found!']);
    end

    desc = load(desc);
    
end


%%%%
%some sanity checks for the important parameters
%%%%

if (~isfield(desc, 'NCond'))
    error('NCond must be given');
end
NCond = desc.NCond;

if (~isfield(desc, 'numWires'))
    disp('genMatCirc: WARNING: assuming number of wires equal to NCond-double(gndPlane)');
    numWires = NCond;
    if (isfield(desc, 'gndPlane'))
        if (desc.gndPlane)
            numWires = NCond-1;
        end
    end
else
    numWires = desc.numWires;
end

if (isfield(desc, 'NDiel'))
    NDiel = desc.NDiel;
else
    NDiel = numWires;
end

if (~isfield(desc, 'condRad'))
    error('descFname needs to contain vector condRad');
end
condRad = desc.condRad;
if (1 == length(condRad))
    condRad = ones(1, numWires)*condRad;
end

if (length(condRad) ~= numWires)
    error('condRad must contain numWires entries');
end



if (~isfield(desc, 'dielThickness'))
    error('descFname needs to contain vector dielThickness');
end
dielThickness = desc.dielThickness;
if (1 == length(dielThickness))
    dielThickness = ones(1, NDiel)*dielThickness;
end

if (length(dielThickness) ~= NDiel)
    error('dielThickness must have NDiel entries');
end



if (~isfield(desc, 'e_r'))
    error('descFname needs to contain array with dielectric permittivities');
end
e_r = desc.e_r;
if (1 == length(e_r))
    e_r = e_r * ones(1, NDiel);
end

if (length(e_r) ~= NDiel)
    error('first dimension of e_r must match number of dielectrica');
end

%some sanity checks for more or less optional parameters
if (~isfield(desc, 'condPos'))
    error('descFname needs to contain conductors position');
else
    condPos = desc.condPos;
end
if (size(condPos, 1) ~= numWires)
    error('condPos must have numWires rows');
end
if (~isfield(desc, 'dielPos'))
    if (DEBUG)
        disp('genMatCirc: assuming dielPos is equal to condPos');
    end
    dielPos = condPos;
else
    dielPos = desc.dielPos;
end

if(size(dielPos,1) ~= NDiel)
    error('dielPos must have NDiel rows');
end


if (~isfield(desc, 'dielIRad'))
    if (DEBUG)
        disp('genMatCirc: assuming dielIRad is equal to condRad');
    end
    dielIRad = condRad;
else
    dielIRad = desc.dielIRad;
end

if (1 == length(dielIRad))
    dielIRad = ones(1, NDiel)*dielIRad;
end

if (length(dielIRad) ~= NDiel)
    error('dielIRad must have NDiel entries');
end

dielORad = dielIRad + dielThickness;


NCircCond = numWires;
gndPlane = false;
if (isfield(desc, 'gndPlane'))
    gndPlane = desc.gndPlane;
end

if (gndPlane)
    numWires = numWires + 1;
    if (~isfield(desc, 'gndWidth') || ~isfield(desc, 'gndHeight'))
        error('if gndPlane is true, gndWidht and gndHeight need to be specified');
    end
    gndWidth = desc.gndWidth;
    gndHeight = desc.gndHeight;
end

%a ground plane is always the reference conductor.
if (~gndPlane)
    if (~isfield(desc, 'gndIdx') && ~isfield(desc, 'refC'))
        disp('genMatCirc: WARNING: No reference conductor set. Using the first as reference.');
        refC = 1;
    else
        if (isfield(desc, 'gndIdx'))
            refC = desc.gndIdx;
        else
            refC = desc.refC;
        end
    end
else
    %ground plane always is the first conductor
    refC = 1;
end

%%%%
%load problem region
%%%%
if (~isfield(desc, 'xmin'))
    error('descFname needs to contiain xmin of problem region');
end
xmin = desc.xmin;
if (~isfield(desc, 'xmax'))
    error('descFname needs to contiain xmax of problem region');
end
xmax = desc.xmax;
if (~isfield(desc, 'ymin'))
    error('descFname needs to contiain ymin of problem region');
end
ymin = desc.ymin;
if (~isfield(desc, 'ymax'))
    error('descFname needs to contiain ymax of problem region');
end
ymax = desc.ymax;


%%%%
%check if conductors and dielectricas are in the problem region
%%%%

mostLeftCond  = min(condPos(:, 1) - condRad');
mostRightCond = max(condPos(:, 1) + condRad');
mostDownumWires  = min(condPos(:, 2) - condRad');
mostUpCond    = max(condPos(:, 2) + condRad');

mostLeftDiel  = min(dielPos(:, 1) - dielORad');
mostRightDiel = max(dielPos(:, 1) + dielORad');
mostDownDiel  = min(dielPos(:, 2) - dielORad');
mostUpDiel    = max(dielPos(:, 2) + dielORad');

if (xmin > min(mostLeftCond,mostLeftDiel))
    disp('genMatCirc: WARNING: image size may be wrong');
    disp(['most left point on conductor : ' num2str(mostLeftCond)]);
    disp(['most left point on dielectric: ' num2str(mostLeftDiel)]);
    disp(['xmin: ' num2str(xmin)]);
end
if (xmax < max(mostRightCond,mostRightDiel))
    disp('genMatCirc: WARNING: image size may be wrong');
    disp(['most right point on conductor : ' num2str(mostRightCond)]);
    disp(['most right point on dielectric: ' num2str(mostRightDiel)]);
    disp(['xmax: ' num2str(xmax)]);
end
if (ymin > min(mostDownumWires, mostDownDiel))
    disp('genMatCirc: WARNING: image size may be wrong');
    disp(['lowest point on conductor : ' num2str(mostDownumWires)]);
    disp(['lowest left point on dielectric: ' num2str(mostDownDiel)]);
    disp(['ymin: ' num2str(ymin)]);
end
if (ymax < max(mostUpCond, mostUpDiel))
    disp('genMatCirc: WARNING: image size may be wrong');
    disp(['highest point on conductor : ' num2str(mostUpCond)]);
    disp(['highest point on dielectric: ' num2str(mostUpDiel)]);
    disp(['ymax: ' num2str(ymax)]);
end

%%%%
%calculate grid
%%%%

%unfortunately, the fdm algorithm used is not very stable for extremly
%large number of nodes. Too small number of nodes would decrease accuracy
%beyond usefulness, therefore we limit the total number of nodes here.

MAXTOTNUMNODES = 200000;
MINTOTNUMNODES = 10000;

tf = false;
tooBig = false;
tooSmall = false;
while(~tf)
    s = 1/nodesPerUnit;

    %coordinates of grid nodes.
    x = xmin:s:xmax;
    y = ymin:s:ymax;

    %number of rows and columns
    numR = length(y);
    numC = length(x);

    numNodes = numR*numC;
    
    if (numNodes > MAXTOTNUMNODES)
        tooBig = true;
        nodesPerUnit = nodesPerUnit/1.05;
    elseif (numNodes < MINTOTNUMNODES)
        tooSmall = true;
        nodesPerUnit = nodesPerUnit*1.05;
    else
        tf = true;
    end
end

if (tooBig)
    disp(['genMatCirc: WARNING: number of nodes too big (>' ...
            num2str(MAXTOTNUMNODES) ')']);
end
    
if (tooSmall)
    disp(['genMatCirc: WARNING: number of nodes too small (<' ...
            num2str(MINTOTNUMNODES) ')']);
end




%this will create two vectors, X, and Y, where every pair of
%values from the vectors (X(i), Y(i)) is one node. Every node
%in the mesh is represented by one pair of values in these two vectors.
%Each vector therefore contains numNodes entries.
[X Y] = meshgrid(x,y);

%red green and blue color channels for conductor image
imR = 255*ones(numR, numC);
imG = 255*ones(numR, numC);
imB = 255*ones(numR, numC);

h = figure;
colors = colormap(hsv(numWires+NDiel));
close(h);

if (~isequal(size(colors, 1), size(unique(colors, 'rows'), 1)))
    error('wat? didn''t return unique colors');
end

condCol = colors(1:numWires, :); %colors if the input shall be plotted
dielCol = colors(numWires+1:end, :);

%compute the images for every dielectric
E_R = ones(numR, numC);
imSum = zeros(numR, numC);
for i = 1:NDiel
    centre = dielPos(i, :);
	dist = sqrt((centre(1)-X).^2 + (centre(2)-Y).^2);
    dielIm = logical((dist <= dielORad(i)) .* (dist > dielIRad(i)));
    E_R(dielIm) = e_r(i);
    imSum = imSum + double(dielIm);
    
    imR(dielIm) = dielCol(i, 1);
    imG(dielIm) = dielCol(i, 2);
    imB(dielIm) = dielCol(i, 3);
end

if (max(max(imSum)) > 1)
    if (PLOTINPUT)
        imshow(imSum)
    end
    error('check dielectrics, they seem to be overlappying');
end

%compute the images for every conductor, plus one for the ground plane if
%needed
cIm = cell(numWires, 1);

for i = 1:NCircCond
    j = i + double(gndPlane);
    centre = condPos(i, :);
    dist = sqrt((centre(1)-X).^2 + (centre(2)-Y).^2);
	condIm = false(numR, numC);
	condIm( dist <= condRad(i) ) = true;
	cIm{j} = condIm;
    imSum = imSum + condIm;
    
    imR(condIm) = condCol(j, 1);
    imG(condIm) = condCol(j, 2);
    imB(condIm) = condCol(j, 3);
end

if (gndPlane)
    condIm = false(numR, numC);
%     condIm(numR-4:numR-1, 2:numC-1) = true;
    numHeightNodes = ceil(nodesPerUnit*gndHeight);
    numWidthNodes = ceil(nodesPerUnit*gndWidth);
    
    if (numHeightNodes > numR)
        numHeightNodes = numR-2;
    end
    if (numWidthNodes > numC)
        numWidthNodes = numC-2;
    end
    
    minRowIdx = numR-numHeightNodes;
    maxRowIdx = numR-1;
    minColIdx = ceil((numC-numWidthNodes)/2);
    maxColIdx = floor((numC+numWidthNodes)/2);
    
    minRowIdx = max(2, minRowIdx);
    maxRowIdx = min(numR-1, maxRowIdx);
    minColIdx = max(2, minColIdx);
    maxColidx = min(numC-1, maxColIdx);
    
    if ( ((maxRowIdx - minRowIdx + 1) < 3 ) || ((maxColidx - minColIdx + 1) < 3) )
        disp('genMatCirc: WARNING: gnd specified, the size or the resolution is too small');
    end
    
    condIm(minRowIdx:maxRowIdx, minColIdx:maxColIdx) = true;
    cIm{1} = condIm;
    imSum = imSum + condIm;
    
    imR(condIm) = condCol(1, 1);
    imG(condIm) = condCol(1, 2);
    imB(condIm) = condCol(1, 3);
end

if (max(max(imSum)) > 1)
    imshow(imSum);
    error('something is overlapping!');
end

%
%plot the input data
%
if (PLOTINPUT)
    im = zeros(numR, numC, 3);
    im(:,:,1) = imR;
    im(:,:,2) = imG;
    im(:,:,3) = imB;
    try
        figure, imshow(im);
    catch exception
        disp('genMatCirc: failed to display conductor image');
        disp(exception.message);
    end
    if (ischar(desc))
        title(strrep(desc, '_', ' '));
    else
        title([num2str(numWires) ' conductors']);
    end
end
	
	
end