function V = bbrfdm(V, mask, e_r, gamma, beta, omega, numIt, errMax)
%BBRFDM computes the potential on a mesh based on BBR-FDM
%
% SYNOPSIS V = bbrfdm(V, mask, e_r, gamma, beta, omega, numIt, errMax)
%
% INPUT V: original potential distribution.
%       mask: mask that determines wether a value in V is fixed(1)
%			 or unknown(0)
%       gamma: error constant
% OPTIONAL:
%       beta: overrelaxiation factor for the "big" loop
%       omega: overrelaxiation factor for thte "small" loop (The small loop
%           is the FDM with Dirichlet-condition on the border)
%       numIt: maximum number of iterations in the "big" loop
%       errMax: maximum allowed estimated error
%       
% OUTPUT V: the computed potential distribution
%

DEBUG = false;

if (~exist('beta', 'var'))
    beta = 1;
end
if (~exist('omega', 'var'))
    omega = 1;
end

if (~exist('errMax', 'var'))
    errMax = 1e-6;
end

if (~exist('numIt', 'var'))
    numIt = 1e4;
end

[M N] = size(V);

%TODO: check if the algorithm is valid for uneven M,N
% if ( (1 == mod(M,2)) || (1 == mod(N,2)) )
%     error('bbrfdm: Only even number of mesh points currently supported');
% end

Q = getShiftMatrix(M,N);

[S1rows, S1cols] = getS1Coord(M,N);
[S2rows, S2cols] = getS2Coord(M,N);


% S1rows = [ 2*ones(1, ceil(N/2)-1) 3:(M-2) (M-1)*ones(1,N-2) (M-2):-1:3 2*ones(1, floor(N/2)-1) ];
% S1cols = [ (floor(N/2)+1):(N-1) (N-1)*ones(1, M-4) (N-1):-1:2 2*ones(1, M-4) 2:floor(N/2) ];
% 
% S2rows = [ ones(1, ceil(N/2)-1) 2:(M-1) M*ones(1, N-2) (M-1):-1:2 ones(1, floor(N/2)-1) ];
% S2cols = [ (floor(N/2)+1):(N-1) N*ones(1, M-2) (N-1):-1:2 ones(1, M-2) 2:floor(N/2) ];

% S1rows = [ 2*ones(1, (N/2)-1) 3:(M-2) (M-1)*ones(1,N-2) (M-2):-1:3 2*ones(1, (N/2)-1) ];
% S1cols = [ ((N/2)+1):(N-1) (N-1)*ones(1, M-4) (N-1):-1:2 2*ones(1, M-4) 2:(N/2) ];
% 
% S2rows = [ ones(1, (N/2)-1) 2:(M-1) M*ones(1, N-2) (M-1):-1:2 ones(1, (N/2)-1) ];
% S2cols = [ ((N/2)+1):(N-1) N*ones(1, M-2) (N-1):-1:2 ones(1, M-2) 2:(N/2) ];
% 

s1ind = sub2ind(size(V), S1rows, S1cols);
s2ind = sub2ind(size(V), S2rows, S2cols);

V_S2 = V(s2ind');

delta = 1;
for k = 1:numIt
    
    %compute new values according to Dirichlet-Condition
    V = fdmDirichlet(V, mask, e_r, omega, numIt, errMax/10);

    %get values on S1
    V_S1 = V(s1ind');
    
    %compute new border values for S2 according to values on S1
    nV_S2 = Q*V_S1;
    
    %assign new border values, plus overrelaxation
    V_S2 = V_S2 + beta*(nV_S2 - V_S2); 
    
    delta = max(abs(V(s2ind') - V_S2));
    
    err = delta*(gamma/(1-gamma));
    
    if (DEBUG)
        fprintf('\b\b\b\b\b\b\b\b\b\b%10.4e', err);
    end
    
    if (err < errMax)
        break;
    end
    
    if (delta > 4)
        error('looks like outer loop is diverging :(');
    end
   
    %update the Dirichlet-Condition for the FDM
    V(s2ind') = V_S2;
end

if (DEBUG)
    fprintf('\n');
end

%check if convergence was reached
if (err > errMax)
    disp('bbrfdm: WARNING: Did not converge on big loop, remaining delta:');
    disp(delta);
end

end
