function q = compCharge(pot, condContour, E_R)
%COMPCHARGE computes the charge on a conductor based on the potential distribution
%
% SYNOPSIS q = compCharge(pot, condContour, E_R)
%
% INPUT pot: potential distribution mesh
%		condContour: the contour of the conductor inside the mesh of the
%			potential distribution. Make sure the contour does not contain
%			mesh points on the edges or corners, as the algorithm will 
%			probably fail in that case
%		E_R: mesh with the relative permittivity of the materials. The 
%			size of the mesh needs to be the same as the one of pot.
%			Make sure that conductors have the same relative permittivity
%			assigned to their contour as the dielectric outside.
%
% OUTPUT q: total free charge on the conductor, given the potential
%				distribution and relative permittivities.
% TODO: this function only works if the vector returned by bwboundaries is
%       walking the boundary in clockwise direction?
%


v_0 = 2.997925e8;
EPS0 = ( v_0 * v_0 * 4e-7 * pi )^(-1); %permittivity in free space

% EPS0 = 8.854187817620e-12;

%controls verbosity
VERBOSE = false;


%
%Some sanity checks
%

%pot is supposed to be a matrix containing the potentials at the nodes.
%contour is a n x 2-vector containing the (x,y)-positions of the nodes
%where the conductor's contour is located.

if (size(condContour,2) ~= 2)
    error(['unexpected size of conductors contour: '...
        num2str(size(condContour))]);
end

if (size(condContour,1) < 5)
    error('there must be at least 4 points on the contour');
end

if (size(E_R) ~= size(pot))
	error(['expected potential grid and permittivity grid to have the '...
		'same size']);
end

%
%Compute starting values
%

%contour, as a series of vectors
vecCC = condContour(2:end, :) - condContour(1:end-1, :);

%vector that led us to the current point
prevVec = vecCC(end, :);
%current point
currPoi = condContour(1, :);
%currPoi: row and column of current point

%total charge on this conductor's surface
q = 0;

%
% Walk around the contour, computing the derivate along
%

for i = 1:size(vecCC, 1)
    %vector that originates in currPoi and points to the next point
    currVec = vecCC(i, :);
    %potential at current point
    currPoiPot = pot(currPoi(1),currPoi(2));
        
%     if (VERBOSE)
%         disp(['P0(' num2str(y(currPoi(1))) ',' num2str(x(currPoi(2))) ')' ...
%             ' = ' num2str(currPoiPot)]);
%     end
    
    %Check in which direction we're going. This is of importance because
    %the calculation of the charge must be adapted if and edge is
    %encountered in the contour.
    %If the current vector is equal to the previous, no change of direction
    %occured.
    %If we're going right, then the current vector would be rotated by
    %-90 degrees compared to the previous vector.
    rightVec = [prevVec(2) -prevVec(1)];
    %If we're going left, then the current vector would be
    %rotated by 90 degrees compared to the previous vector.
    leftVec = [-prevVec(2) prevVec(1)];

    if (currVec == prevVec)
        %potVec is the vector from the current point to the point where the
        %potential is to be determined for subtraction from the conductors
        %potential
        potVec = [-currVec(2) currVec(1)]; %rotation by 90 degrees
        %cartesian coordinates of that point
        potPoi = currPoi + potVec;
        %calculate contribution to total charge
        dq = (currPoiPot - pot(potPoi(1),potPoi(2))) * ...
            E_R(potPoi(1), potPoi(2));
        q = q + dq;
    elseif (isequal(currVec,rightVec)) %we're going right
            
        %It is assumed that the contour is going clockwise, therefore if we
        %are going right, we must consider two outer points for this one
        %point on the conductor
        
        potPoi1 = currPoi + [-prevVec(2) prevVec(1)];
        potPoi2 = currPoi + [-currVec(2) currVec(1)];
        
        dq1 = (currPoiPot - pot(potPoi1(1), potPoi1(2))) * ...
            E_R(potPoi1(1), potPoi1(2));
        dq2 = (currPoiPot - pot(potPoi2(1), potPoi2(2))) * ...
            E_R(potPoi2(1), potPoi2(2));
        q = q + (dq1 + dq2);
    elseif (isequal(currVec,leftVec)) %we're going left
        %if we're going left then the current point
        %on the conductor has no counterpart.
    elseif (prevVec == -currVec) %we're going backwards
        potPoi1 = currPoi + [-prevVec(2) prevVec(1)];
        potPoi2 = currPoi + prevVec;
        potPoi3 = currPoi + [-currVec(2) currVec(1)];
        
        dq1 = (currPoiPot - pot(potPoi1(1), potPoi1(2))) * ...
            E_R(potPoi1(1), potPoi1(2));
        dq2 = (currPoiPot - pot(potPoi2(1), potPoi2(2))) * ...
            E_R(potPoi2(1), potPoi2(2));
        dq3 = (currPoiPot - pot(potPoi3(1), potPoi3(2))) * ...
            E_R(potPoi3(1), potPoi3(2));
%         
%         q = q + dq1 + dq2 + dq3;
%         
        disp('Warning: walking backwards here, do not assume this yields correct results!');
        disp(['Current point: ' num2str(currPoi) '(no ' num2str(i) ')']);
        disp(['previous vector: ' num2str(prevVec)]);
        disp(['current vector: ' num2str(currVec)]);
    else %we're going...???
        disp(prevVec);
        disp(currVec);
        error('wat');
    end 
    %advance one point
    currPoi = currPoi + currVec;
    prevVec = currVec;
end

q = q * EPS0;

end%function
