function PHI = fdmDirichlet(PHI, mask, E_R, omega, numIt, deltaMax)
%FDMDIRICHLET performs a fdm with dirichlet condition on PHI
%
% SYNOPSIS PHI = fdmDirichlet(PHI, mask, E_R, omega, numIt, deltaMax)
%
% INPUT PHI: original potential distribution grid
% 		mask: 1s denote that the potential given in PHI is fixed,
%				0s denote that the potential may be altered
%		E_R: grid with the relative permittivities of the materials
%
% OPTIONAL  omega: overrelaxiation factor for the "big" loop
%			numIt: maximum number of iterations for the "big" loop
%			deltaMax: maximum delta. If delta drops blow this value,
%				convergence is assumed
%
% OUTPUT PHI: computed potential distribution
%
% PHI, mask, E_R all must be matrices of the same size.
% omega, numIt and deltaMax are assumed to be scalars.
%
% The algorithm only works if the borders of the material 
% inhomogenities are between nodes.
%
% The algorithm works with the Dirichlet boundary condition, 
% therefore the boundaries are assumed to be fixed, even if 
% indicated otherwise by mask.

%
% sanity checks
%

if (size(PHI) ~= size(mask))
	error('PHI and mask must be of the same size');
end
if (size(mask) ~= size(E_R))
	error('E_R must have the same size as mask and PHI');
end

%
% set default value for missing input arguments
%

if (~exist('numIt', 'var'))
    numIt = 1e4;
end

if (~exist('deltaMax', 'var'))
    deltaMax = 1e-6;
end

if (~exist('omega', 'var'))
    omega = 1;
end

[numR numC] = size(PHI);

deltaOld = inf;
%outer boundaries are assumed to be fixed (Dirichlet condition)
for it = 1:numIt
    delta = 0;
    for i = 2:numR-1
        for j = 2:numC-1
            if (0 == mask(i,j))
%                 R = ( PHI(i-1,j) + PHI(i+1,j) + PHI(i, j-1) + PHI(i,j+1)) - ...
%                     4 * PHI(i,j);

                %K_ij = e_r(j)/e_r(i);
                %N: i-1,j
                %S: i+1,j
                %W: i,j-1
                %E: i,j+1
                %O: i,j
                
  
                K_ON = E_R(i-1,j)/E_R(i,j);
                K_OS = E_R(i+1,j)/E_R(i,j);
                K_OW = E_R(i,j-1)/E_R(i,j);
                K_OE = E_R(i,j+1)/E_R(i,j);
                
                if (1 == mask(i-1,j))
                    K_ON = 1;
                end
                if (1 == mask(i+1,j))
                    K_OS = 1;
                end
                if (1 == mask(i,j-1))
                    K_OW = 1;
                end
                if (1 == mask(i,j+1))
                    K_OE = 1;
                end
                
                V_N = PHI(i-1,j);
                V_S = PHI(i+1,j);
                V_W = PHI(i,j-1);
                V_E = PHI(i,j+1);
                V_O = PHI(i,j);
                
                R = 2*V_N*K_ON/(1+K_ON) + V_O*(1-K_ON)/(1+K_ON) + ...
                    2*V_E*K_OE/(1+K_OE) + V_O*(1-K_OE)/(1+K_OE) + ...
                    2*V_S*K_OS/(1+K_OS) + V_O*(1-K_OS)/(1+K_OS) + ...
                    2*V_W*K_OW/(1+K_OW) + V_O*(1-K_OW)/(1+K_OW) - ...
                    4*V_O;
                    

                PHI(i,j) = PHI(i,j) + (omega*R/4);
                delta = max(delta, abs(omega*R/4));
            end
        end
    end
    
    %check if convergence is reached
    if (delta < deltaMax)
        break;
    end
    
    %check for divergence
    if (delta > 10*deltaOld)
        disp('fdmDirichlet: WARNING: Inner loop may be diverging');
    end
    deltaOld = delta;
end

%check if we converged or aborted due to maximum number of iterations
if (delta > deltaMax)
    disp('fdmDirichlet: WARNING: inner loop did not converge. Remaining delta: ');
    disp(delta);
    error('fdmDirichlet:divergence', 'Inner loop did not converge');
end

end
