function preCompPsi = genPreCompPsi(nMax)
%PRECOMPPSI computes Psi (see compPsi.m) for m,n<nMax
%
% SYNOPSIS preCompPsi = genPreCompPsi(nMax)
%
% INPUT nMax: Maximum distance in each direction
%
% OUTPUT preCompPsi: upper triangular matrix with precomputed psi values.
%			values for m,n are located at m+1,n+1
%
% Only a triangular matrix in the fourth quadrant will be computed, as
% Psi(m,n) = Psi(n,m) = Psi(n,-m)
% Due to the above relationship, one can also see that neumann condition
% holds at all values that have either n or m equal to zero (in this case,
% this means the upper and left border). The main diagonal serves as the
% right border, using dirichlet condition computed with the formula for m=n
% which is directly computable, and the lower border is computed using an
% approximation formula for r = sqrt(m^2+n^2) -> inf, also imposing
% dirichlet condition.

preCompPsi = zeros(nMax);
sum = 1;
%populate main diagonal
for i = 1:nMax
    m = i-1;
    sum = sum + 1/((2*m)-1);
    preCompPsi(i,i) = 1 - (sum/pi);
end

%populate right border
for i = 1:nMax
    m = i-1;
    n = nMax-1;
    psi = 1 - (1/(4*pi)) * ( log(8*(m*m + n*n)) + 2*0.57721566490153286060 );
    preCompPsi(i, nMax) = psi;
end

%execute fdm, neumann condition at upper, dirichlet at right border
delta = 1;
fprintf('          ');
while(delta > 1e-10)
    delta = 0;
    %impose neumann condition on upper border without edges
    for j = 2:nMax-1
        nPsi = .25*(preCompPsi(1, j-1) + ...
            preCompPsi(1, j+1) + 2*preCompPsi(2, j));
        delta = max(delta, abs(preCompPsi(1, j) - nPsi));
        preCompPsi(1, j) = nPsi;
    end

    %compute other values
    for i = 2:nMax-1
        for j = (i+1):nMax-1
            nPsi = .25 * (preCompPsi(i-1,j) + preCompPsi(i+1,j) + ...
                preCompPsi(i,j-1) + preCompPsi(i,j+1));
            delta = max(delta, abs(preCompPsi(i,j) - nPsi));
            preCompPsi(i,j) = nPsi;
        end
    end
    fprintf('\b\b\b\b\b\b\b\b\b\b%10.4e', delta);
end
fprintf('\n');
