function R = genResMat(f, descFname)
% GENRESMAT computes the resistance matrices for solid and stranded wires
%
% SYNOPSIS R = genResMat(f, descFname)
%
% INPUT f: frequencies at which to compute the resistance matrix
%       descFname: file name where description of the conductors is found
% OUTPUT R: resistance matrices: (numWires-1 x numWires-1 x length(f)).
%
% The description file must contain the following variables:
%   - numWires: number of conductors
%   - condRad: radii of conductors (1 x numWires)
%   - sigma: conductivities of conductors (1 x numWires)
% The following variables are optional in the description file:
%   - refC: reference conductor. Defaults to 1 if omitted (scalar). 
%   - stranded: true if conductor is stranded (1 x numWires). Defaults to
%   false if omitted
%   - numOuter: number of strands on outer ring. (1 x sum(stranded))
%   - numStrand: total number of strands. (1 x sum(stranded))
%       Note: If you have the radii of the outer strands, set this to
%       condRad^2/strandRad^2.

%note: the value for 16 strands is a pure guess, as this is a configuration
%where the wire is not concentric stranded, but bunch stranded -> Wires are
%twisted without a geometric relationship to each other and there is no
%outer 'ring' on which the number of strands could be determined.
strandTable = struct(...
    'numStrand', ...
    {1 7 16 19 37}, ...
    'numOuter', ...
    {1, 6, 11, 12, 18});

DEBUG = false;

%see Clayton R. Paul: Analysis of Multiconductor
%transmission lines, page 155.
%a factor that is needed to estimate the resistance of rectangular
%conductors
RESFACT = 1.4;
MU0 = 4*pi*1e-7;

if (isstruct(descFname))
    file = descFname;
else
    if (~exist(descFname, 'file') && ~exist([descFname '.mat'], 'file'))
        error(['genResMat: description file ''' descFname  '''not found!']);
    end

    file = load(descFname);

end

if (~isfield(file, 'NCond'))
    error('NCond must be given');
end
NCond = file.NCond;

if (~isfield(file, 'numWires'))
    disp('genResMat: WARNING: assuming number of wires equal to NCond-double(gndPlane)');
    numWires = NCond;
    if (isfield(file, 'gndPlane'))
        if (file.gndPlane)
            numWires = NCond-1;
        end
    end
else
    numWires = file.numWires;
end

if (~isfield(file, 'condRad'))
    error('genResMat: descFname must contain condRad');
end
condRad = file.condRad;
if (1 == length(condRad))
    condRad = ones(1, numWires)*condRad;
end
if (length(condRad) ~= numWires)
    error('condRad must either have one entry, or numWires. Note: GND isn''t countet in numWires');
end

if (~isfield(file, 'sigma'))
    error('genResMat: descFname must contain sigma');
end
sigma = file.sigma;
if (1 == length(sigma))
    sigma = ones(1, numWires)*sigma;
end
if (length(sigma) ~= numWires)
    error('sigma must either have one entry, or numWires. Note: GND isn''t countet in numWires');
end
if (~isfield(file, 'refC'))
    if (DEBUG)
        disp('genResMat: WARNING: No reference conductor specified, using the first conductor as reference');
    end
    refC = 1;
else
    refC = file.refC;
end

if ( (refC > numWires) || (refC < 1) )
    error('genResMat: 0 < refC <= numWires');
end

if (~isfield(file, 'stranded'))
    if (DEBUG)
        disp('genResMat: WARNING: Assuming all conductors are solid core conductors.');
    end
    stranded = false(1, numWires);
else
    stranded = file.stranded;
end

if (sum(stranded) > 0)
    if (~isfield(file, 'numStrand'))
        error('genResMat: descFname must contain total number of strands');
    end
    numStrand = file.numStrand;
    
    %we assume that numStrand has as many entries as there are stranded
    %wires. If numStrand has as many entries as there are conductors,
    %discard the entries that belong to nonstranded conductors (they should
    %be zero or one anyway)
    if (length(numStrand) == length(stranded))
        numStrand = numStrand(stranded);
    end
    
    if (length(numStrand) ~= sum(stranded))
        error(['genResMat: ' num2str(sum(stranded)) ' stranded wires but number of strands given for ' num2str(length(numStrand))]);
    end
       
    if (~isfield(file, 'strandRad'))
        error('genResMat: descFname must contain radius of strands');
    end
    strandRad = file.strandRad;
    if (length(strandRad) == length(stranded))
        strandRad = strandRad(stranded);
    end
    if (length(strandRad) ~= sum(stranded))
        error(['genResMat: ' num2str(sum(stranded)) ' stranded wires but number of strands given for ' num2str(length(strandRad))]);
    end
    
    %check if the number of outer strands is given
    if (isfield(file, 'numOuter'))
        n_o = file.numOuter;
    else
        n_o = zeros(1, sum(stranded));
    end
    
    %check if the number of outer strands is given for the conductors that
    %are not stranded, too
    if (length(n_o) == length(stranded))
        n_o = n_o(stranded);
    end
    
    %check if n_o has as many entries as there are stranded conductors
    if (length(n_o) ~= sum(stranded))
        error('genResMat: for every stranded conductor the number of outer strands must be given');
    end
    
    %check if some of the entries of n_o are zero, and if they are, try to
    %guess the number of outer strands
    for k = 1:sum(stranded)
        if (0 ~= n_o(k))
            continue;
        end
        cfg = [strandTable.numStrand] == numStrand(k);
        if (1 ~= sum(cfg))
            error(['genResMat: number of outer conductors in stranded wire not given, and unknown for given number of total strands: ' num2str(numStrand(k))]);
        end
        n_o(k) = strandTable(cfg).numOuter;
    end
end

NCircCond = numWires;
gndPlane = false;
if (isfield(file, 'gndPlane'))
    gndPlane = file.gndPlane;
    if (gndPlane)
        if (~isfield(file, 'gndWidth') || ~isfield(file, 'gndHeight') ||  ~isfield(file, 'gndSigma'))
            error('if gndPlane is true, gndWidth, gndHeight, and gndSigma must be given');
        end
        gndWidth = file.gndWidth;
        gndHeight = file.gndHeight;
        gndSigma = file.gndSigma;
    end
end

nEle = NCond-1;

%for every frequency, one R matrix
R = zeros(nEle, nEle, length(f));

for k = 1:length(f)
    freq = f(k);
    r = zeros(1, NCircCond);
    
    x_c = ones(1, NCircCond); %correction factor for stranded wires
    
    delta = 1./sqrt(pi*MU0*freq*sigma); %skin depth
    
    if (0 ~= sum(delta > condRad))
        error('genResMat: the algorithm is not suited for such low frequencies!');
    end
    
    if (sum(stranded) > 0)
        % formula used by tonello in his paper. note there is a pi missing in the
        % paper.
        x_c(stranded) = n_o .* ( ...
            acos((strandRad - delta(stranded))./strandRad) .* (strandRad.^2) - ...
            (strandRad - delta(stranded)).*sqrt(strandRad.^2 - (strandRad - delta(stranded)).^2) ...
            ) ./ (2*condRad(stranded).*delta(stranded)*pi);
    end
   
    for c = 1:NCircCond
        if (delta(c) > 2*condRad(c))
            r(c) = 1/(sigma(c)*pi*condRad(c)*condRad(c));
        else
            r(c) = sqrt( (MU0*freq)/(4*sigma(c)*pi*condRad(c)*condRad(c)) );
        end
    end
    
    r = r./x_c;
    
    %add ground plane, if neccessary
    if (gndPlane)
        w = gndWidth;
        t = gndHeight;
        delta = 1/sqrt(pi*MU0*freq*gndSigma);
        if (delta > (w*t)/(RESFACT*(w+t)))
            gndRes = 1/(gndSigma*w*t);
        else
            gndRes = 1/(RESFACT*gndSigma*delta*(w+t));
        end
        r = [gndRes r];
    end
    
    R(:,:,k) = r(refC);
    notRefC = setdiff(1:length(r), refC);
    R(:,:,k) = R(:,:,k) + diag(r(notRefC));
    
end

end
