function Q = getShiftMatrix(M, N)
%GETSHIFTMATRIX returns shift matrix for grid of size (M,N)
%
% SYNOPSIS Q = getShiftMatrix(M,N)
%
% INPUT M,N: size of the grid for which the shift should be generated
%
% OUTPUT Q: shift matrix
%
%
% An entry Q_{ij} in the shift matrix indicates the potential difference 
% between the i-th and the j-th node. The linear numbering for the nodes
% start at the most left point on the top edge for both S1 and S2. To get
% the coordinates of the points, getSxCoord returns two vectors
% containing row and column numbers of the nodes, where the first entry
% is associated with the first (most left point on the top edge) point.
%
% The alogrithm will save computed matrices for given mesh sizes as 
% 'Q_MxN.mat' and try to load those files, if existant. Also it will
% try to load precomputed values of psi, which is used in computing 
% the potential difference between two points.

%
%Number of values for psi that should be precomputed.
%Values for larger indices will be computed using the formula for r->inf
%
NUMPRECOMPPSI = 100;

%
%Where the shift matrices are stored
%
QDIR = 'tmp/shiftmatrices/';

%only works if N,M are even numbers
% if (0~=mod(M,2))
%     error('M must be an even number');
% end
% if (0~=mod(N,2))
%     error('N must be an even number');
% end

if (~exist(QDIR, 'dir'))
	mkdir('tmp/shiftmatrices/');
end
QFName = [QDIR 'Q_' num2str(M) 'x' num2str(N) '.mat'];
if (exist(QFName, 'file'))
    %fprintf('getShiftMatrix: Found precomputed shift matrix in directory, loading...');
    Qstruct = load(QFName);
    Q = Qstruct.Q;
    %fprintf('Done\n');
    return;
end

%disp('getShiftMatrix: Couldn''t find precomputed shift matrix in directory, generating ...');

PsiFName = [QDIR 'preCompPsi_' num2str(NUMPRECOMPPSI) '.mat'];
%PsiFName = 'preCompPsi_6.mat';
if (exist(PsiFName, 'file'))
    %fprintf('getShiftMatrix: Found precomputed potential values, loading...');
    pcPsi = load(PsiFName);
    preCompPsi = pcPsi.preCompPsi;
    %fprintf('Done\n');
else
    %fprintf('getShiftMatrix: Couldn''t find precomputed potential values, generating ...');
    preCompPsi = genPreCompPsi(NUMPRECOMPPSI);
    save(PsiFName, 'preCompPsi');
    %fprintf('Done\n');
end

%Compute matrix S that gives the potentials on S1 raised by sources on S1
[S1rows, S1cols] = getS1Coord(M,N);

numS1Points = length(S1rows);

S = zeros(numS1Points);
%compute influences of points on S1 on each other
for i = 1:numS1Points
    for j = 1:numS1Points
        m = S1rows(i) - S1rows(j);
        n = S1cols(i) - S1cols(j);
        S(i,j) = getPsi(m,n, preCompPsi);
    end
end

%Compute matrix P that gives potentials on S2 raised by sources on S1
[S2rows, S2cols] = getS2Coord(M,N);

numS2Points = length(S2rows);
P = zeros(numS2Points, numS1Points);
for i = 1:numS2Points
    for j = 1:numS1Points
        m = S2rows(i) - S1rows(j);
        n = S2cols(i) - S1cols(j);
        P(i,j) = getPsi(m,n, preCompPsi);
    end
end

%compute shift matrix
Q = P/S;

save(QFName, 'Q');

%disp('getShiftMatrix: Done.');

end
