function [C L G1] = mtlPUL(cIm, E_R, refC)
%MTLPUL computes PUL parameters for a MTL line
%
% SYNOPSIS [C L G1] = mtlPUL(cIm, E_R, refC)
%
% INPUT cIm: (1xN) cell array of matrices, each matrix of the same size. N
%        is the total number of conductors.
%        Each matrix is a "picture" of one conductor (i.e, if the matrix is
%        1 at a point, we assume there is the conductor, and if it is zero
%        at one point, we assume there is no conductor there.)
%       E_R: matrix the same size as the conductor matrices, with each
%       entry denoting the relative permittivity in that place.
%       refC: ID of the reference conductor. cIm{refC} therefore is the
%       image of the reference conductor.
% OUTPUT: C L G1: capacitance, inductance and conductance matrices for
% \omega = 1.
%

%should the computation of the potential field be carried out in parallel
%for each conductor?
CONDPARALLEL = true;
%should the conductors be smoothened?
%this will avoid having single outstanding pixels that will irritate compCharge
SMOOTHEN = true;
%should debugging information be printed?
DEBUG = true;
%should progress information be printed? (doesn't work with parfor)
PROGRESS = false;
if (CONDPARALLEL)
    PROGRESS = false;
end
%maximum number of workers, should be minimum of number of CPU cores and
%what your license allows
MAXCLUSTERSIZE = 4;

MU0 = 4*pi*1e-7;
EPS0 = 8.854187817620e-12;

%note: this will cause problems if the number of nodes is too small, 
%because then a conductor might be erased by
%imerode. We do this because single outstanding pixels in the conductors
%will cause problems later on with compCharge.
if (SMOOTHEN)
    if (DEBUG)
        disp('Smoothening conductors');
    end
    for k = 1:length(cIm)
        try
            im = imerode(cIm{k}, ones(3));
        catch exception
            disp('mtlPUL: INFO: failed to smoothen conductors:');
            disp(exception.message);
            break;
        end
        
        if (0 == sum(sum(im)))
            disp('either increase number of nodes per unit or disable smoothening of conductors.');
            disp('note: disablign smoothening of conductors may cause other problems');
            error('mtlPUL:condTooSmall', ['conductor no ' num2str(k) ' is to small.']);
        end
        
        try
            cIm{k} = imdilate(im, ones(3));
        catch exception
            disp('mtlPUL: INFO: failed to smoothen conductors:');
            disp(exception.message);
            break;
        end
    end
end

%number of conductors
NCond = size(cIm, 1);
%number of rows, columns
[numR, numC] = size(E_R);
%number of nodes
numNodes = numR*numC;

%the mask will specify where the potential is fixed (at the conductors),
%and where it can change
mask = zeros(numR, numC);
for i = 1:NCond
	mask = mask + cIm{i};
end

if (sum(sum(mask > 1)) > 0)
	error('conductors are overlapping... that won''t work');
end

if (DEBUG)
    disp(['got mesh with ' num2str(numR) 'x' num2str(numC) ...
        ' = ' num2str(numNodes) ' nodes']);
end

%%%
%generation of input parameters done
%%%

%starup the matlab worker pool
%TODO: find out maximum cluster size
if ( (0 == matlabpool('size')) && CONDPARALLEL)
    %maximum cluster size. should be minium of maximum cluster size
    %according to license and number of cores on the machine.
    poolSize = min(MAXCLUSTERSIZE, 2*NCond);
    if (DEBUG)
        disp(['opening matlab pool with ' num2str(poolSize) ' workers']);
    end
    try
        matlabpool(poolSize);
    catch exception
        disp('mtlPUL: INFO: failed to start matlabpool.');
        disp(exception.message);
    end
        
end

t = cos(pi/numC) + sin(pi/numR);
omega = (8 - sqrt(64 - 16*t*t)) / (t*t);
    
%%%
%create some parameters for error estimation at current resolution
%%%
Verr = zeros(numR, numC);
Verr([1 end], 2:(end-1)) = 1;
Verr(2:(end-1), [1 end]) = 1;
Verr = fdmDirichlet(Verr, mask, ones(size(mask)), 1, 1e10, 1e-6);
Qerr = getShiftMatrix(numR, numC);
alpha1 = max(max(Verr(2:(end-1), 2:(end-1))));
alpha2 = max(sum(Qerr,2));
gamma = alpha1*alpha2;
if (DEBUG)
    disp(['upper bound on row-sum: ' num2str(alpha1) 'x' num2str(alpha2)...
        '=' num2str(gamma)]);
end
if (gamma >= 1)
    disp('I don''t think this will converge');
    error('mtlPUL:divergence', 'shiftmatrix malformed');
end
    
%%%
%start computation
%%%

CGEN = zeros(NCond, 2*NCond); %generalized capacitance matrix

%for every conductor, set the conductors potential to 1 and all
%others potential to 0, then compute the potentials at the free
%points

parfor k = 1:(2*NCond) %conductor that has unit potential
    %commit the acutal computation of potentials on the grid nodes
    
    if (k > NCond)
        V = bbrfdm(double(cIm{k-NCond}), mask, ones(size(E_R)), gamma, 1., omega, 1e6, 1e-6);
    else
        V = bbrfdm(double(cIm{k}), mask, E_R, gamma, 1., omega, 1e6, 1e-6);
    end
    
    %compute charge on every conductor for the current scenario
    q = zeros(2*NCond, 1);
    for i = 1:NCond
        condCont = bwboundaries(cIm{i}, 4, 'noholes');
        %bwboundaries: Each cell contains a Q x 2 matrix, where Q is the
        %number of boundary pixels for the corresponding region. Each row
        %of these Q x 2 matrices conatins the row and column coordinates of
        %a boundary pixel.
        if (length(condCont) ~= 1)
            disp(['mtlPUL: unable to determine contour of conductor ' ...
                num2str(i)]);
            figure, imshow(cIm{i});
            title('mtlPUL: image of conductor that causes trouble');
            error('bwboundaries returned no or more than one boundary');
        end
        if (k > NCond)
            q(i+NCond) = compCharge(V, condCont{1}, ones(size(E_R)));
        else
            q(i) = compCharge(V, condCont{1}, E_R);
        end
    end
    
    if (k > NCond)
        CGEN(:,k) = q(NCond+1:end);
    else
        CGEN(:,k) = q(1:NCond);
    end
    
    if (PROGRESS)
      printf('.');
    end
    
end

%compute capacitance matrix
CapC = compCap(CGEN(1:NCond, 1:NCond), refC);
Cap0 = compCap(CGEN(1:NCond, NCond+1:end), refC);
%C(i,j): charge on i-th conductor due to unit voltage on j-th conductor


C = real(CapC);
L = MU0*EPS0*inv(Cap0);
G1 = -imag(CapC);

end
